/***************************************************************************
 *   Copyright (C) 2004 by Johnny Andersson                                *
 *   johnny_a@sourceforge.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CONV_H
#define CONV_H

#include <qobject.h>
#include <qvaluelist.h>
#include "dictionary.h"

/**
@author Johnny Andersson
*/
// Simple temporary representation (used for returning entries from parseLine)
class KanjiTemp
{
public:
    uchar strokes;
    uchar grade;
    ushort freq;
    ushort unicode;
    ushort jis;
    QCString kunReadings;
    QCString onReadings;
    QCString nameReadings;
    QCString meanings;
    QCString radicals;
    int pad;
};

// Simple temporary representation
class DictEntryTemp
{
public:

    bool operator<(const DictEntryTemp& d)
         { return (d.reading > reading); };

    QByteArray markers;
    QCString jpn;
    QCString reading;
    QCString english;
    int pad;
};

class Conv : public QObject
{
    Q_OBJECT
public:
    enum ConvertErrors {OK, NoCodec, NoSourceFile, NoDestFile, NoRadicalFile, NoMarkerFile, NoWFreqFile};

    Conv(QObject *parent = 0, const char *name = 0);
    ~Conv();

    ConvertErrors readWFreqFile(const QString& fileName);
    
    ConvertErrors convertKanjiDicFile(const QString& sourceFileName, const QString& radFileName, const QString& destFileName, const QString& codecName, const QString& radCodecName);
    ConvertErrors convertFile(const QString& sourceFileName, const QString& destFileName, Dictionary::DictFormats dictFormat, const QString& codecName, const QString& markerFileName);

private:
    QMemArray<int> createCharIndex(QValueList<DictEntryTemp>& entries);
    QMemArray<int> createKanjiIndex(QValueList<DictEntryTemp>& entries);
    QMap<QString, QString> _wfreq;
    KanjiTemp parseKanjiDicLine(const QString& line);
    DictEntryTemp parseEdictLine(const QValueList<Marker>& markers, const QString& line);
};

#endif
