/***************************************************************************
 *   Copyright (C) 2004 by Johnny Andersson                                *
 *   johnny_a@sourceforge.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <iostream>
#include "conv.h"

using std::cout;


void usage()
{
    cout << "\nDictionary converter for Narau (narau.sourceforge.net). Usage:\nnarauconv type infile outfile radical_file/marker_file wordfreq_file\ntype is kanjidic or edict. Infiles have to use eucJP encoding except radical_file (utf8). \".bin\" and \".meta\" will be added to outfile\nwordfreq_file only needed for edict\n" << endl;
}

int main(int argc, char **argv)
{
    std::setbuf(stdout, 0);
    std::setbuf(stderr, 0);

    QString codec("eucJP");
    QString type;
    QApplication a(argc, argv);
    Conv::ConvertErrors err;

    if(a.argc() > 1)
        type = QString(a.argv()[1]);

    if((type != "kanjidic" && type != "edict") ||
        (type == "kanjidic" && a.argc() != 5) ||
        (type == "edict" && a.argc() != 6))
    {
        usage();
        exit(1);
    }
    
    Conv conv;

    if(type == "kanjidic")
        err = conv.convertKanjiDicFile(a.argv()[2], a.argv()[4], a.argv()[3], codec, "utf8");
    else if(type == "edict")
    {
        cout << "Reading wordfreq file...\n";
        err = conv.readWFreqFile(a.argv()[5]);
        if(err == Conv::OK)
            err = conv.convertFile(a.argv()[2], a.argv()[3], Dictionary::Edict, codec, a.argv()[4]);
    }
    
    if(err == Conv::OK)
    {
        cout << "Success\n";
        exit(0);
    }

    else if(err == Conv::NoCodec)
        cout << "Codec doesn't exist" << endl;
    else if(err == Conv::NoSourceFile)
        cout << "No such source file" << endl;
    else if(err == Conv::NoDestFile)
        cout << "Couldn't open destination file" << endl;
    else if(err == Conv::NoRadicalFile)
        cout << "Couldn't open radical file" << endl;
    else if(err == Conv::NoMarkerFile)
        cout << "Couldn't open marker file" << endl;
    else if(err == Conv::NoWFreqFile)
        cout << "Couldn't open wfreq file" << endl;

    return 1;
}
