/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if defined(USE_QPE)
  #include <qpe/qpeapplication.h>
#else
  #include <qapplication.h>
#endif

#include <iostream>
#include <qpushbutton.h>
#include <qdir.h>
#include "znarau.h"
#include "jpnlineedit.h"
#include "convert.h"
#include "dictmanager.h"
#include "listmanager.h"
#include "kanjidic.h"
#include "hcr.h"
#include "configmanager.h"
#include "fonts.h"

using std::cerr;

int main( int argc, char ** argv )
{
	QString base;

#if defined(USE_QPE)
    QPEApplication a(argc, argv);
#else
    QApplication a(argc, argv, "znarau");
#endif

    setbuf(stdout, 0);
    setbuf(stderr, 0);

    QString errors;

    glFonts = new Fonts();
    glKanjiDic = new KanjiDic();
    glDictManager = new DictManager();
    glListManager = new ListManager();
    glConfigManager = new Config("JARDS Mobile");
    glConfigManager->setGroup("General");


#if defined(USE_QPE)
	base = QString((((QPEApplication*)qApp)->qpeDir()+"jardsmob/"));
#else
	base = QDir::homeDirPath() + "/jardsmob/";
#endif

    glFonts->setImagePath(base + "images/");

    JpnLineEdit::initialize(base + "romkana.utf8");
    glDictManager->loadMarkers(base + "markers");
    glKanjiDic->loadRadicals(base + "raddat.utf8");
    glKanjiDic->setStrokeDataFile(base + "strokedata");
    glKanjiDic->setFile(base + "kanjidic.jardskdic");
    HCR::initialize("kpengine", base + "strokedata");
    glDictManager->addFile(base + "edict.jardsdic");
//    glDictManager->addFile(base + "enamdict.bin");

    ZNarau mw;

#if defined(USE_QPE)
    a.showMainWidget(&mw);
#else
    a.setMainWidget(&mw);
    mw.show();
#endif

    a.exec();

	delete glConfigManager;

}
