/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <qtabwidget.h>
#include <qaction.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qtextbrowser.h>
#include <qobjectlist.h>
#include <qmessagebox.h>
#include <qwidgetlist.h>
#include <qlabel.h>
#include "configmanager.h"
#include "mainwidget.h"
#include "itemdetails.h"
#include "editlists.h"
#include "search.h"
#include "item.h"
#include "listmanager.h"
#include "globaleventfilter.h"
#include "fontdialog.h"
#include "itemlistview.h"
#include "aboutdialogbase.h"
#include "waitwidget.h"
#include "settingsdialog.h"

using std::cerr;


MainWidget::MainWidget(QWidget *parent, const char *name, WFlags f)
        : MainWidgetBase(parent, name, f)
{



    connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(slotBeforeExit()));

    QObjectList *list = queryList("ItemPopup");
    QObjectListIt *it = new QObjectListIt(*list);
    QObject * obj;
    while ((obj=it->current()) != 0)
    {
        ++(*it);
        connect((ItemPopup *)obj, SIGNAL(focusSearch(QChar)), __search, SLOT(slotFocusSearch(QChar)));

        connect((ItemPopup *)obj, SIGNAL(setTheseRadicals(QChar)), this, SLOT(slotSetTheseRadicals(QChar)));

        connect((ItemPopup *)obj, SIGNAL(addToList(const Item *)), glListManager, SLOT(slotAddToCurrentList(const Item *)));

        connect((ItemPopup *)obj, SIGNAL(showItem(const Item *)), this, SLOT(slotShowItem(const Item *)));

    }
	delete list;
	delete it;
    
    list = queryList("EditLists");
    it = new QObjectListIt(*list);
    while ((obj=it->current()) != 0)
    {
        ++(*it);
        connect((EditLists *)obj, SIGNAL(showItem(const Item *)), this, SLOT(slotShowItem(const Item *)));
    }
    delete list;
	delete it;

    list = queryList("Results");
    it = new QObjectListIt(*list);
    while ((obj=it->current()) != 0)
    {
        ++(*it);
        connect((EditLists *)obj, SIGNAL(showItem(const Item *)), this, SLOT(slotShowItem(const Item *)));

        connect((ItemPopup *)obj, SIGNAL(addToList(const Item *)), glListManager, SLOT(slotAddToCurrentList(const Item *)));
    }
    delete list;
	delete it;

    GlobalEventFilter *gef = new GlobalEventFilter();
    qApp->installEventFilter(gef);
    connect(gef, SIGNAL(focusSearch()), this, SLOT(slotFocusSearch()));

    glListManager->loadAll();
    __editLists->reset();

	glFonts->restoreSettings();
	glFonts->emitSignals();

	qApp->setFont(glFonts->font(Fonts::FONT_Generic), true);

	glConfigManager->setGroup("General");
	QStringList ilist = QStringList::split(",", glConfigManager->readEntry("BoxIntervals"));
	if(ilist.count() == 5)
	{
		for(int x=0; x<5; x++)
			Box::setRepeatDelay(x, ilist[x].toInt());
	}
}

MainWidget::~MainWidget()
{
}

void MainWidget::slotSettings()
{
	SettingsDialog *dia = new SettingsDialog();
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	dia->exec();
	delete dia;	
}

void MainWidget::slotShowItem(const Item *item)
{
    __details->slotShowItem(item);

    __mainTab->setCurrentPage(1);
    __details->setFocus();

}

void MainWidget::slotFocusSearch()
{
    __mainTab->showPage(SearchPage);
    __search->slotFocusSearch();
}

void MainWidget::slotFocusSearch(QChar ch)
{
    __mainTab->showPage(SearchPage);
    __search->slotFocusSearch(ch);
}

void MainWidget::slotBeforeExit()
{
	glConfigManager->setGroup("General");
	glConfigManager->writeEntry("BoxIntervals", QString("%1,%2,%3,%4,%5").arg(Box::repeatDelay(0)).arg(Box::repeatDelay(1)).arg(Box::repeatDelay(2)).arg(Box::repeatDelay(3)).arg(Box::repeatDelay(4)));
	
	
	glFonts->saveSettings();
}

void MainWidget::slotSetTheseRadicals(QChar ch)
{
    __mainTab->setCurrentPage(0);
    __search->slotSetTheseRadicals(ch);
}

void MainWidget::slotHelpIndex()
{
}

void MainWidget::slotContextHelp()
{
}

void MainWidget::slotAbout()
{
	AboutDialogBase *dia = new AboutDialogBase(this, 0, true);
//	dia->__about->setText("<p>JARDS Mobile version " + _version + ", Copyright (C) " + _year + " " + "Johnny Andersson<br/>" + dia->__about->text());
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	dia->exec();
	delete dia;	
}

void MainWidget::slotSetFonts()
{
	FontDialog *dia = new FontDialog(this);
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	if(dia->exec())
	{
		dia->apply();
		qApp->setFont(glFonts->font(Fonts::FONT_Generic), true);
		glFonts->emitSignals();
	}
	delete dia;

	QObjectList list = *children();
	QObject *obj;

/*	for(obj = list.first(); obj != 0; obj = list.next())
	{
		if(obj->inherits("ItemPopup"))
	}
*/

}

void MainWidget::slotSetIcons()
{
	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)
	{
		emit iconSizeChanged(Fonts::ICONSIZE_Large);
		glFonts->setIconSize(Fonts::ICONSIZE_Large);
	}
	else 
	{
		emit iconSizeChanged(Fonts::ICONSIZE_Small);
		glFonts->setIconSize(Fonts::ICONSIZE_Small);
	}

	glFonts->emitSignals();

}


