/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qdir.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qwidgetstack.h>
#include <qmenubar.h>
#include <waitwidget.h>
#ifdef USE_QPE
  #include <qpe/qpeapplication.h>
#else
  #include <qapplication.h>
#endif
#include "progressbar.h"
#include "znarau.h"
#include "hcr.h"
#include "mainwidget.h"
#include "fonts.h"
#include "listmanager.h"
#include "helpbrowserdialog.h"
using std::cerr;


ZNarau::ZNarau( QWidget* parent,  const char* name, WFlags fl ) : QMainWindow(parent, name, fl)
{
	setCaption("JARDS Mobile");


	MainWidget *w = new MainWidget(this);

	connect(w, SIGNAL(iconSizeChanged(Fonts::IconSizes)), this, SLOT(slotIconSizeChanged(Fonts::IconSizes)));

	_settingsMenu = new QPopupMenu;
	QPopupMenu *helpMenu = new QPopupMenu;
	QPopupMenu *fileMenu = new QPopupMenu;

//	helpMenu->insertItem("Context help...", w, SLOT(slotContextHelp()));
	helpMenu->insertItem("Help...", this, SLOT(slotHelpIndex()));
	helpMenu->insertItem(tr("About..."), w, SLOT(slotAbout()));

	_settingsMenu->setCheckable(true);
	_settingsMenu->insertItem(tr("Fonts..."), w, SLOT(slotSetFonts()));
	_largeIconsId = _settingsMenu->insertItem(tr("Large icons"), w, SLOT(slotSetIcons()));
	_settingsMenu->insertItem(tr("Configure..."), w, SLOT(slotSettings()));
	
//	fileMenu->insertItem(tr("Installed modules..."), this, SLOT(slotInstalledModules()));
//	fileMenu->insertItem(tr("Dictionaries..."));

	fileMenu->insertItem(tr("Quit"), qApp, SLOT(quit()));
	menuBar()->insertItem(tr("File  "), fileMenu);
	menuBar()->insertItem(tr("Settings  "), _settingsMenu);
	menuBar()->insertItem(tr("Help  "), helpMenu);

	setCentralWidget(w);

	if(glFonts->iconSize() == Fonts::ICONSIZE_Large)
		_settingsMenu->setItemChecked(_largeIconsId, true);

}

ZNarau::~ZNarau()
{}

void ZNarau::slotIconSizeChanged(Fonts::IconSizes size)
{
	if(size == Fonts::ICONSIZE_Small)
		_settingsMenu->setItemChecked(_largeIconsId, false);
	else
		_settingsMenu->setItemChecked(_largeIconsId, true);

}

void ZNarau::closeEvent(QCloseEvent *e)
{

	WaitWidget *w = new WaitWidget(this);
	w->move(0,0);
	w->resize(width(), height());
	w->__clock->setPixmap(QPixmap(glFonts->imagePath() + "clock.png"));
	w->show();
	qApp->processEvents();
    glListManager->saveAll();
	QWidget::closeEvent(e);

}

void ZNarau::slotInstalledModules()
{
/*
	InstalledModulesDialogBase *dia = new InstalledModulesDialogBase(this, 0, true);


//	dia->__about->setText(

	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	dia->exec();
	delete dia;	
*/
}

void ZNarau::slotHelpIndex()
{
	QString file;
#if defined(USE_QPE)
	file = QString((((QPEApplication*)qApp)->qpeDir()+"/jardsmob/help/tutorial/index.html"));
#else
	file = QDir::homeDirPath() + "/jardsmob/help/tutorial/index.html";
#endif

	HelpBrowserDialog *dia = new HelpBrowserDialog();
	dia->loadFile(file);
	dia->showMaximized();
	dia->exec();

	delete dia;
	
}


