/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qtextstream.h>
#include <qfile.h>
#include "convert.h"

using std::cerr;


Convert::Convert()
{
    _initialized = false;
}

Convert::~Convert()
{
}

QString Convert::romajiToKana(const QString& str, bool hiragana)
{
    // FIXME: Only works if called after each new character added to str

    if(!_initialized)
        return "";

    QString ret = str;
    QString r,*k;
    uint i,pos=0;
    int changed=0;

    QDict<QString> *dict;

    if(hiragana)
        dict = _hiraganaDict;
    else
        dict = _katakanaDict;


    while (pos<ret.length())
    {
        for (i=3;i>0;i--)
        {
            r=ret.mid(pos,i);
            if ((k=dict->find(r))!=0)
            {
                ret.replace(pos,i,*k);
                pos+=i;
                changed=1;
                break;
            }
        }

        if (i==0)
        {
            if ((ret[pos]>QChar('A'))&&(ret[pos]<QChar('z')))
            {
                if ((ret.at(pos)==QChar('n'))&&(pos!=ret.length()-1))
                {
                    ret.replace(pos,1,QString("q"));
                    changed=1;
                }
                if (ret[pos]==ret[pos+1])
                {
                    ret.replace(pos,1,QString("!"));
                    changed=1;
                }
            }
            pos++;
        }
    }

    return ret;

}

bool Convert::initialize(const QString& romKanaFileName)
{
    int n=0,i,j;
    QDict<QString> *kanD = NULL;
    QString kan;

    _hiraganaDict = new QDict<QString>(223,false);
    _katakanaDict = new QDict<QString>(223,false);
    _hiraganaDict->setAutoDelete(true);
    _katakanaDict->setAutoDelete(true);

    QFile f(romKanaFileName);

    if ( f.open(IO_ReadOnly) )
    {
        QTextStream t(&f);
        t.setEncoding(QTextStream::UnicodeUTF8);
        QString s;
        while ( !t.eof() )
        {
            s = t.readLine();

            if (s.left(1)==QChar('#'))
                continue;

            if (s.left(9)=="$HIRAGANA")
            {
                n=0;
                kanD=_hiraganaDict;
                continue;
            }

            if (s.left(9)=="$KATAKANA")
            {
                n=0;
                kanD=_katakanaDict;
                continue;
            }
            i=s.find(' ')+1;
            j=s.find(' ',i);
            if (j==-1) j=s.length();
            kan = s.left(i-1);
            kanD->insert(s.mid(i,j-i).stripWhiteSpace(),new QString(kan));
            n++;
        }
        f.close();
        _initialized = true;
        return true;
    }

    return false;


}

