/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** This file was originally created for the Kiten project (www.katzbrown.com/kiten)
** by Jazon Katz-Brown (jason@katzbrown.com)
** Adapted to JARDS by Johnny Andersson (andersson.johnny@gmail.com)
*/


#include <iostream>
#include <qstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmap.h>
#include <qregexp.h>
#include <qstringlist.h>
#include <qtextcodec.h>
#include <qvaluelist.h>
#include "deinf.h"

using std::cerr;


Deinf::Index::Index()
{
    loaded = false;
}

void Deinf::Index::load()
{
    if (loaded)
        return;

    QString vconj = "   vconj";

    QFile f(vconj);

    if (!f.open(IO_ReadOnly))
    {
        cerr << "Couldn't open \"vconj\"";
        return;
    }

    QTextStream t(&f);
    t.setCodec(QTextCodec::codecForName("utf8"));

    for(QString text = t.readLine(); !t.eof() && text.at(0) != '$'; text = t.readLine())
    {
        if(text.at(0) != '#')
        {
            unsigned int number = text.left(2).stripWhiteSpace().toUInt();
            QString name = text.right(text.length() - 2).stripWhiteSpace();

            names[number] = name;
        }
    }
    for(QString text = t.readLine(); !text.isEmpty(); text = t.readLine())
    {
        if(text.at(0) != '#')
        {
            QStringList things(QStringList::split(QChar('\t'), text));

            Conjugation conj;
            conj.ending = things.first();
            conj.replace = (*things.at(1));
            conj.num = things.last().toUInt();

            list.append(conj);
        }
    }

    f.close();
    loaded = true;
}

namespace
{
    QStringList possibleConjugations(const QString &text)
    {
        QStringList endings;
        for (unsigned i = 0; i < text.length(); ++i)
            endings.append(text.right(i));
        return endings;

    }
}

QStringList Deinf::Index::deinflect(QString text, QStringList &name)
{
    load();
    QStringList ret;

    for(QValueListIterator <Conjugation> it = list.begin(); it != list.end(); ++it)
    {
        if(text.right(strlen((*it).ending)) == (*it).ending)
        {
            name.append(names[(*it).num]);

            //kdDebug() << "match ending: " << (*it).ending << "; replace: " << (*it).replace << "; name: " << names[(*it).num] << endl;

            QString tmp(text);
            tmp.replace(QRegExp((*it).ending + "*", false, true), (*it).replace);
            ret.append(tmp);
//            cout << tmp.utf8() << endl;

        }
    }

    return ret;
}
/*
QStringList Deinf::Index::deinflect(QString text, QStringList &name)
{
    load();
    QStringList endings = possibleConjugations(text);
    QStringList ret;

    for (QValueListIterator <Conjugation> it = list.begin(); it != list.end(); ++it)
    {
        QStringList matches(endings.grep(QRegExp(QString("^") + (*it).ending)));

        if (matches.size() > 0) // a match
        {
            name.append(names[(*it).num]);

            //kdDebug() << "match ending: " << (*it).ending << "; replace: " << (*it).replace << "; name: " << names[(*it).num] << endl;

            QString tmp(text);
            tmp.replace(QRegExp((*it).ending + "*", false, true), (*it).replace);
            ret.append(tmp);
            cout << tmp.utf8() << endl;

        }
    }

    return ret;
}

  */
