/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DICTIONARY_H
#define DICTIONARY_H

#ifdef USE_QPE
  #include <qlist.h>
#else
  #include <qptrlist.h>
#endif
    
#include <qdict.h>
#include <qfile.h>
#include <qstring.h>
#include <qmap.h>
#include <qcstring.h>
#include <qvaluelist.h>
#include <qobject.h>
#include "itemlist.h"
#include "deinf.h"

class Marker 
{

public:
    Marker() {};
    Marker(const QString& m, const QString& e) : marker(m), expl(e) {};
    QString marker;
    QString expl;

};

// The "header" before the strings in memory/disk
class DictWordStorage 
{
public:
    // The lengths
    ushort markersLen;
    ushort jpnLen;
    ushort readingLen;
    ushort englishLen;

    // Immediately following in memory: the actual strings in utf8 format, zero terminated
};

class MMapFile 
{
public:
	int _kanaIndexSize;

	uint _wordOffs;
	uint _kanaOffs;
	uint _kanjiOffs;
    void *_mmapBase;
    QFile _qfile;

};

class DictWord 
{

public:

    DictWord();

    QByteArray markers() const;
    const CString jpn() const;
    const CString reading() const;
    const CString english() const;

	QString infoString() const;

    void setPointer(DictWordStorage *ptr) {_ptr = ptr;};
    DictWordStorage *ptr() {return _ptr;};

private:
    DictWordStorage *_ptr;

};


class Dictionary : public QObject
{
    Q_OBJECT
public:

    enum DictFormats {Edict,Enamdict};
    enum SearchMode {Exact, Beginning, End, Middle, Regexp};

    Dictionary(const QValueList<Marker>& markers);
    Dictionary(const QValueList<Marker>& markers, const QString& fileName);
    ~Dictionary();

    bool setFile(const QString& fileName);
	bool Dictionary::setKanjiIndexFile(const QString& fileName);

    int entries() {return _numEntries;};

    void setSearchMode(SearchMode mode) {_searchMode = mode;};
    SearchMode searchMode() {return _searchMode;};

    void J2E(const QString& j, QList<const DictWord> *res);
    void E2J(const QString& e, QList<const DictWord> *res);

    void cancelSearch();
    bool searchCancelled() {return _cancelled;};

signals:
    void searchProgress(const QString&, float);

private:
	QArray<int> getKanjiIndices(const QChar& kanji);
	void getKanaRange(QChar ch, int *first, int *last) const;
    uint _numEntries;
    QString _name;
    DictWord *_entries;
	int *_kanjiIndexBase;
    QValueList<Marker> _markers;
	MMapFile _file;
    bool _cancelled;
    SearchMode _searchMode;
    Deinf::Index _index;

};

#endif
