/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qtextstream.h>
#include <qdatastream.h>
#include <qtextcodec.h>
#include "dictmanager.h"

using std::cerr;

DictManager::DictManager() : QObject()
{
    _searchMode = Dictionary::Exact;
    _dicts.setAutoDelete(true);
    _cancelled = false;
}

DictManager::~DictManager()
{
    _dicts.clear();
    _markers.clear();
}

uint DictManager::totalEntries()
{
    uint count = 0;

    for(uint i = 0; i < _dicts.count(); i++)
        count += _dicts.at(i)->entries();

    return count;
}

bool DictManager::addFile(const QString& fileName)
{
    Dictionary *d = new Dictionary(_markers);

	connect(d, SIGNAL(searchProgress(const QString&, float)), this, SIGNAL(searchProgress(const QString&, float)));

    bool res;

    res = d->setFile(fileName);

    if(res == true)
        _dicts.append(d);
    else
        delete d;

    return res;

}

QStringList DictManager::markers(const QByteArray& m) const
{
    QStringList ret;

    for(int i=0; i < m.count(); i++)
        ret.append(_markers[m[i]].marker);

    return ret;
}

QString DictManager::markerDescription(const QString& id)
{
    for(int i=0; i<_markers.count(); i++)
    {
        if(_markers[i].marker == id)
            return _markers[i].expl;
    }

    return "";
}

bool DictManager::loadMarkers(const QString& fileName)
{
    QFile f(fileName);
    QString str;
    QStringList strlist;

    _markers.clear();

    if ( f.open(IO_ReadOnly) )
    {
        QTextStream t(&f);
        t.setEncoding(QTextStream::UnicodeUTF8);
        while (!t.eof())
        {
            str = t.readLine();
            if(str[0] == '#' || str.stripWhiteSpace().length() <= 2)
                continue;
            strlist = strlist.split('\t',str);
            _markers.append(Marker(strlist[0], strlist[1]));
        }
        f.close();
        return true;
    }
    else
        return false;
}

void DictManager::searchAllJ2E(const QString& j, QList<const DictWord> *res)
{
    QListIterator<Dictionary> it(_dicts);
    Dictionary *dict;

    for(; (dict = it.current()) != 0; ++it)
    {
        dict->setSearchMode(_searchMode);
        dict->J2E(j, res);
    }

}

void DictManager::searchAllE2J(const QString& e, QList<const DictWord> *res)
{
    
    QListIterator<Dictionary> it(_dicts);
    Dictionary *dict;

    for(; (dict = it.current()) != 0; ++it)
    {
       dict->setSearchMode(_searchMode); 
       dict->E2J(e, res);
    }

}

void DictManager::cancelSearch()
{
    QListIterator<Dictionary> it(_dicts);
    Dictionary *dict;

    for(; (dict = it.current()) != 0; ++it)
    {
       dict->cancelSearch();
    }
	
}

QValueList<Marker> DictManager::markers()
{
    return _markers;
}

DictManager *glDictManager = 0;
