/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DICTMANAGER_H
#define DICTMANAGER_H

#ifdef USE_QPE
  #include <qlist.h>
#else
  #include <qptrlist.h>
#endif
#include <qcstring.h>
#include <qstring.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include "dictionary.h"

class DictManager : public QObject 
{
    Q_OBJECT
public:

    DictManager();
    ~DictManager();

    QStringList markers(const QByteArray& m) const;

    bool busy() {return _busy;};

    Dictionary& getDictionary(int n) {return *_dicts.at(n);};
    uint dictionaries() const {return _dicts.count();};
    uint totalEntries();

    Dictionary::SearchMode searchMode() {return _searchMode;};
    void setSearchMode(Dictionary::SearchMode mode) {_searchMode = mode;};
    void searchAllJ2E(const QString& j, QList<const DictWord> *res);
    void searchAllE2J(const QString& e, QList<const DictWord> *res);
    void cancelSearch();
    bool searchCancelled() {return _cancelled;};

    bool addFile(const QString& fileName);
    bool loadMarkers(const QString& fileName);
    QString DictManager::markerDescription(const QString& id);
    QValueList<Marker> markers();

signals:
    void searchProgress(const QString&, float);

private:
    Dictionary::SearchMode _searchMode;
    bool _busy;
    bool _cancelled;
    QList<Dictionary> _dicts;
    QValueList<Marker> _markers;


};

extern DictManager *glDictManager;

#endif
