/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qmessagebox.h>
#include <qt.h>
#include "itemlist.h"
#include "listmanager.h"

using std::cerr;

ItemList::ItemList() : QObject()
{
	_defaultSkills = (1 << Box::KanjiToKana) | (1 << Box::KanaToKanji) | (1 << Box::KanaToMeaning) | (1 << Box::MeaningToKana);
    _name = QObject::tr("Unnamed");
	_changed = true;
	_locked = false;
}

ItemList::~ItemList()
{
}

// change this to "itemsDue" and use in TrainSelect::slotStartClicked to avoid duplication

int ItemList::dueCount()
{
	int c = 0;

	QListIterator<Item> it(_items);

	for(; it.current(); ++it)
	{
		int skills;
		if(it.current()->box()->skills() & (1 << Box::ListDefaults))
			skills = _defaultSkills;
		else
			skills = it.current()->box()->skills();
		for(int skill=0; skill<6; skill++)
		{
			if(((skill == Box::MeaningToKanji) || (skill == Box::KanaToKanji)|| (skill == Box::KanjiToKana)|| (skill == Box::KanjiToMeaning)) && (it.current()->kanji().length() == 0))
				continue;
			if((skills & (1 << skill)) == 0)
				continue;
			if(it.current()->box()->timeLeft((Box::Skills) skill) < 1)
				++c;
		}
	}

	return c;

}

void ItemList::setName(const QString& name)
{
    _name = name;
	emit changed();
}

void ItemList::setDefaultSkills(int skills)
{
	_defaultSkills = skills; 

	QListIterator<Item> it(_items);

	for(; it.current(); ++it)
	{
		// change the skills for all items that have non-custom skills
		if((it.current()->box()->skills() & (1 << Box::ListDefaults)))
			it.current()->box()->setSkills(skills | (1 << Box::ListDefaults));
	}

	_changed = true;
	emit changed();
}

bool ItemList::save(const QString& fileName, bool alwaysSave)
{
	if((_changed == false) && (alwaysSave == false))
		return true;

    QFile file(fileName);
    QTextStream ts(&file);

    if(!file.open(IO_WriteOnly))
	{
        return false;
	}
    ts.setEncoding(QTextStream::UnicodeUTF8);

    ts << "# JARDS study list v1" << endl;

    ts << "@list" << endl;
    ts << "name=" << _name << endl;
    ts << "default_skills=" << _defaultSkills << endl;

	QListIterator<Item> it(_items);

	for(; it.current(); ++it)
	{
        ts << it.current()->saveString();
	}

    file.close();

	_changed = false;

    return true;
}

bool ItemList::load(const QString& fileName)
{
    QFile file(fileName);
    QTextStream ts(&file);
	QString str;

    if(!file.open(IO_ReadOnly))
        return false;

    ts.setEncoding(QTextStream::UnicodeUTF8);

    while(!file.atEnd())
    {
        QString tmp;
        QStringList slist;

		if(str[0] != '@')
        	str = ts.readLine();

        if(str[0] == '#')
            continue;

        if(str == "@list")
        {
            str = ts.readLine();
            while(str[0] != '@')
            {
                slist = slist.split('=', str);
                tmp = slist[0];
                if(tmp == "name")
                {
                    setName(slist[1]);
                }
                else if(tmp == "default_skills")
                {
                    _defaultSkills = slist[1].toInt();
                }
				if(!file.atEnd())
                	str = ts.readLine();
				else
					break;
            }
        }
        else if(str == "@word")
        {
            QStringList strings;

            str = ts.readLine();

            while(str[0] != '@')
            {
                strings += str;
				if(!file.atEnd())
                	str = ts.readLine();
				else
					break;
            }

            Word *w = new Word();
			w->box()->setSkills(_defaultSkills | (1 << Box::ListDefaults));
            w->loadStringList(strings);

            append(w);
        }
        else if(str == "@kanji")
        {
            QStringList strings;

            str = ts.readLine();

            while(str[0] != '@')
            {
                strings += str;
				if(!file.atEnd())
                	str = ts.readLine();
				else
					break;
            }

            Kanji *k = new Kanji();
			k->box()->setSkills(_defaultSkills);
            k->loadStringList(strings);

            append(k);
        }
    }
 
    file.close();

	_changed = false;

    return true;
}


void ItemList::append(Item *item)
{
	_changed = true;

	_items.append(item);

	emit itemAdded();
	emit changed();
}

bool ItemList::remove(int index)
{
	_changed = true;

	bool ret = _items.remove(index);
	emit changed();
	return ret;
}

