/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ITEMLIST_H
#define ITEMLIST_H

#include <qstring.h>
#ifdef USE_QPE
  #include <qlist.h>
#else
  #include <qptrlist.h>
#endif
#include <qobject.h>
#include "item.h"

class ItemList : public QObject
{
	Q_OBJECT
public:
    ItemList();
    ~ItemList();

	// Note: QObject has disabled copy constructor and assignment operator

    QString name() {return _name;};
    void setName(const QString& name);

    bool save(const QString& fileName, bool alwaysSave = false);
    bool load(const QString& fileName);

	void append(Item *item);
	bool remove(int index);
	int count() const {return _items.count();};
	int dueCount();
	void clear() { _items.clear(); };
	Item *operator[](int i) {return _items.at(i);};
	
	void setLocked(bool locked) {_locked = locked;};
	bool locked() const {return _locked;};

	bool changed() const {return _changed;};
	void setChanged(bool changed) {_changed  = changed;};

	int defaultSkills() const {return _defaultSkills;};
	void setDefaultSkills(int skills);

signals:
	void itemAdded();
	void changed();

private:
	int _defaultSkills;
	bool _locked;
	QDateTime _lastEdited;
    QString _name;
	QList<Item> _items;
	bool _changed;
};

#endif
