/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef KANJIDIC_H
#define KANJIDIC_H

#include <qfile.h>
#include <qstring.h>
#include <qintdict.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include "strokedata.h"
#include "cstring.h"

class DictKanjiStorage
{
public:
    ushort strokes;
    ushort grade;
    ushort freq;
    ushort unicode;
    ushort jis;
    ushort kunLen;
    ushort onLen;
    ushort nameLen;
    ushort meaningsLen;
    ushort radicalsLen;

};

class DictKanji
{
public:

    DictKanji();
    ~DictKanji();

    void setPointer(DictKanjiStorage *ptr) {_ptr = ptr;};

    QChar character() const {return QChar(_ptr->unicode);};
    ushort unicode() const {return _ptr->unicode;};
    ushort jis() const{return _ptr->jis;};
    int strokes() const{return _ptr->strokes;};
    int frequency() const {return _ptr->freq;};
    int grade() const  {return _ptr->grade;};

    QStringList onReadingsList() const;
    QStringList kunReadingsList() const;
    QStringList nameReadingsList() const;
    QStringList meaningsList() const;

    const CString onReadings() const;
    const CString kunReadings() const;
    const CString nameReadings() const;
    const CString meanings() const;
    const CString radicals() const;

private:
    DictKanjiStorage    *_ptr;
};

class Kanji;

class KanjiDic
{
public:

    KanjiDic();
    ~KanjiDic();


    static bool isHiragana(QChar ch) {return ch.unicode() >= 0x3040 && ch.unicode() <= 0x309f ? true : false;};
    static bool isKatakana(QChar ch) {return ch.unicode() >= 0x30a0 && ch.unicode() <= 0x30ff ? true : false;};
    static bool isKana(QChar ch) {return isHiragana(ch) || isKatakana(ch);};
    static bool isKanji(QChar ch)
    {
        return (ch.unicode() >= 0x4e00 && ch.unicode() <= 0x9faf)
               || (ch.unicode() >= 0x3400 && ch.unicode() <= 0x4dbf);
    }; 

    static bool isJpn(QChar ch) {return isKana(ch) || isKanji(ch);};
    static bool isParticle(QChar ch) {return ch.unicode() == 0x3092 ? true : false;};  // Only checks for 'wo' object marker


    bool setStrokeDataFile(const QString& fileName);
	const DictKanji *getDictKanji(QChar ch) const;
    Kanji getKanji(QChar ch) const;
    Kanji getKanjiByJIS(ushort jis) const;
    QString getKanjiByStrokes(int strokes, int deviation = 0) const;
    QString getKanjiStrokes(const Kanji *kanji);
    QString getKanjiByRadicals(const QString& rads, int  strokes = 0);
    QString getKanjiByOnReading(const QString& str) const;
    QString getKanjiByNameReading(const QString& str) const;
    QString getKanjiByMeaning(const QString& str) const;

    QString getRadicals(QChar kanji);
    QString getRadicalMeanings(QChar ch) const;
    ushort getRadicalStrokes(QChar ch) const;
    QString getRadicalsByStrokes(int strokes) const;


    int numEntries() const  {return _numEntries;};
    bool setFile(const QString& fileName);
    bool loadRadicals(const QString& fileName, const QString& codecName = "utf8");


private:
    uint _numEntries;
    uint _numRadicals;
    DictKanji *_entries;
    QFile _file;
    StrokeData _sd;
    DictKanjiStorage *_mmapBase;
    QIntDict<QString> _radicals;
    QIntDict<QString> _kanjiToRadicals;
    QIntDict<QString> _strokesToRadicals;
};

extern KanjiDic *glKanjiDic;

#endif
