/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "kanjiquery.h"
#include "kanjidic.h"
#include "item.h"

KanjiQuery::KanjiQuery()
{
    clear();
}


KanjiQuery::~KanjiQuery()
{
}

void KanjiQuery::clear()
{
    _on = "";
    _name = "";
    _meaning = "";
    _radicals = "";
    _strokes = 0;
    _deviation = 0;
}

void KanjiQuery::setStrokes(int strokes, int deviation)
{
    _strokes = strokes;
    _deviation = deviation;
}

void KanjiQuery::setRadicals(const QString& radicals)
{
    _radicals = radicals;
}

void KanjiQuery::setReadings(const QString& on, const QString& name, const QString& meaning)
{
    _on = on;
    _name = name;
    _meaning = meaning;
}

QString KanjiQuery::search() const
{
    QString ret;
    QString tmp;

    if(_strokes > 0)
        ret = glKanjiDic->getKanjiByStrokes(_strokes, _deviation);

    if(_radicals.length())
    {
        if(_strokes > 0)
        {
            for(int i=0; i < ret.length(); i++)
            {
                bool ok = true;
                QString rads = glKanjiDic->getRadicals(ret[i]);
                for(int r=0; r < _radicals.length(); r++)
                {
                    if(rads.contains(_radicals[r]) == 0)
                    {
                        ok = false;
                        break;
                    }
                }

                if(ok)
                    tmp += ret[i];
            }
            ret = tmp;
        }
        else
        {
            ret = glKanjiDic->getKanjiByRadicals(_radicals);
        }
    }

    if(_on.length() > 0)
    {
        if(ret.length() > 0)
        {
            tmp = "";
            for(int i=0; i < ret.length(); i++)
            {
                Kanji k = glKanjiDic->getKanji(ret[i]);

                if((k.onReadingsList().grep(_on)).count() != 0)
                    tmp += ret[i];

            }
            ret = tmp;
        }
        else
            ret = glKanjiDic->getKanjiByOnReading(_on);
    }

    if(_name.length() > 0)
    {
        if(ret.length() > 0)
        {
            tmp = "";
            for(int i=0; i < ret.length(); i++)
            {
                Kanji k = glKanjiDic->getKanji(ret[i]);

                if((k.nameReadingsList().grep(_name)).count() != 0)
                    tmp += ret[i];
            }
            ret = tmp;
        }
        else
            ret = glKanjiDic->getKanjiByNameReading(_name);
    }

    return ret;

}

