/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <qdir.h>
#include "listmanager.h"

#ifdef USE_KDE
    #include <kstandarddirs.h>
#endif

#include <qstringlist.h>
#include "dictmanager.h"
#include "kanjidic.h"

using std::cerr;

ListManager::ListManager() : QObject()
{
    _currentList = -1;
    _lists.setAutoDelete(true);
}

ListManager::~ListManager()
{
}

int ListManager::count()
{
    return _lists.count();
}

ItemList *ListManager::appendList(const QString& name)
{
    ItemList *list = new ItemList();
    list->setName(name);
    _lists.append(list);

	connect(list, SIGNAL(changed()), this, SIGNAL(changed()));

	emit changed();

    return list;
}

ItemList *ListManager::getListByName(const QString& name)
{
    for(int i=0; i<_lists.count(); i++)
	{
		if(_lists.at(i)->name() == name)
			return _lists.at(i);
	}

	return 0;
	
}


ItemList *ListManager::listAt(int index)
{
    return _lists.at(index);
}

QStringList ListManager::listNames()
{
    QStringList ret;

    for(int i=0; i<_lists.count(); i++)
        ret += _lists.at(i)->name();

    return ret;
}

bool ListManager::nameExists(const QString& name)
{
   for(int i=0; i<_lists.count(); i++)
        if(_lists.at(i)->name() == name)
            return true;

    return false;

}

void ListManager::deleteList(ItemList *list)
{
    QString base;
    QDir dir;

#ifdef USE_KDE
    base =  locateLocal("appdata", "lists/");
#else
    base = QDir::homeDirPath() + "/.jards/lists/";
#endif

    QFile file(base + list->name());

    _lists.remove(list);

    file.remove();
}

bool ListManager::saveAll()
{
    QString base;
    QDir dir;

	dir.cd(QDir::homeDirPath());

	if(dir.cd(".jards") == false)
	{
		if(dir.mkdir(".jards") == false)
		{
			return false;
		}
	}

	dir.cd(".jards");

	if(dir.cd("lists") == false)
	{
		if(dir.mkdir("lists") == false)
		{
			return false;
		}
	}

	dir.cd("lists");

    for(int i=0; i<_lists.count(); i++)
    {
        _lists.at(i)->save(dir.absPath() + "/" + _lists.at(i)->name());
    }
}

void ListManager::loadAll()
{
    QStringList::Iterator it;
    QStringList strlist;
    QDir dir;

    if(dir.cd(QDir::homeDirPath() + "/.jards/lists") == false)
		return;

    strlist = dir.entryList(QDir::Files);
    it = strlist.begin();

    for(; it != strlist.end(); ++it)
    {
        ItemList *list = new ItemList();
        list->load(dir.path() + "/" + (*it));
        _lists.append(list);
		connect(list, SIGNAL(changed()), this, SIGNAL(changed()));
    }

	if(_lists.count() > 0)
		_currentList = 0;

	emit doneLoading();
}

void ListManager::slotAddToCurrentList(const Item *item)
{
	if(_currentList == -1)
		return;

	Item *i = item->clone();
	i->box()->setSkills(_lists.at(_currentList)->defaultSkills() | (1 << Box::ListDefaults));
	_lists.at(_currentList)->append(i);
}

ListManager *glListManager = 0;

