/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sys/mman.h>
#include "strokedata.h"

StrokeData::StrokeData()
{
    _mmapBase = 0;
}


StrokeData::~StrokeData()
{
    if(_file.isOpen())
    {
        munmap(_mmapBase, _file.size());
        _file.close();
    }
}

bool StrokeData::initialize(const QString& fileName)
{
    if(_mmapBase != 0)
        return false;

    _file.setName(fileName);

    if(!_file.open(IO_ReadOnly))
    {
        return false;
    }

    _mmapBase = (char *) mmap(0, _file.size(), PROT_READ, MAP_SHARED, _file.handle(), 0);

    if(_mmapBase == MAP_FAILED)
    {
        _mmapBase = 0;
        _file.close();
        return false;
    }

    return true;

}

QString StrokeData::getStrokes(int strokes, ushort jis)
{
    uint offs = 0;
    uint length = 0;
    ushort sjis = 0;
    bool found = false;

    if(_mmapBase == 0)
        return QString("");

    offs = sizeof(uint);

    // Jump to pos in data for kanji with this strokecount

    for(int s=0; s < strokes-1; s++)
    {

        length = ((uint(_mmapBase[offs]) & 0xff) << 24) | ((uint(_mmapBase[offs+1]) & 0xff) << 16)
                 | ((uint(_mmapBase[offs+2]) &0xff) << 8) | (uint(_mmapBase[offs+3]) & 0xff);

        offs += length + sizeof(uint)*2;

    }

    // Search to this kanji

    offs += sizeof(uint);
    sjis = Jis2Sjis(jis);

    while(!found && (offs + 1 < _file.size()))
    {
        if(sjis == (((ushort(_mmapBase[offs]) & 0xff) << 8) | ushort((_mmapBase[offs+1]) & 0xff)))
        {
            // We found the character

            offs += 2;
            int offs2 = offs;

            // Find out where next 2-byte sjis character sequence starts and
            // return the characters inbetween

            while(((_mmapBase[offs2] & 128) == 0) && offs2 < _file.size())
                offs2++;

            QString ret;
            for(int i=0; i<offs2-offs; i++)
                ret += _mmapBase[offs+i];
            return ret;

            found = true;
        }
        offs++;
    }

    return "";

}


// This function taken from "CJKV Information Processing" by Ken Lunde
// Modified by Johnny Andersson

ushort StrokeData::Jis2Sjis(ushort jis)
{
    uchar c2 = (uchar) jis & 0xff;
    uchar c1 = (uchar) ((jis & 0xff00) >> 8);

    int rowOffset = c1 < 95 ? 112 : 176;
    int cellOffset = c1 % 2 ? (c2 > 95 ? 32 : 31) : 126;

    uchar r1 = c1;
    uchar r2 = c2;

    r1 = ((c1 + 1) >> 1) + rowOffset;
    r2 += cellOffset;

    return (uint)r2 | (uint)r1 << 8;
}
