/*
    JARDS Mobile - part of JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qpopupmenu.h>
#include <qcombobox.h>
#include <qtoolbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qtextview.h>
#include <qapplication.h>
#include "hypertextbrowser.h"
#include "boxtrain.h"
#include "listmanager.h"
#include "fonts.h"
#include "scribble.h"

BoxTrain::BoxTrain(QWidget *parent, const char *name)
        : BoxTrainWidget(parent, name)
{

    setFocusPolicy(StrongFocus);

	connect(__unknown, SIGNAL(clicked()), this, SLOT(slotUnknownClicked()));
	connect(__known, SIGNAL(clicked()), this, SLOT(slotKnownClicked()));
	connect(__clue, SIGNAL(clicked()), this, SLOT(slotClueClicked()));
	connect(__answer, SIGNAL(clicked()), this, SLOT(slotAnswerClicked()));
	connect(__ignore, SIGNAL(clicked()), this, SLOT(slotIgnoreClicked()));
	connect(__finish, SIGNAL(clicked()), this, SLOT(slotFinishClicked()));
//	connect(__pad, SIGNAL(clicked()), this, SLOT(slotPadClicked()));

	connect(glFonts, SIGNAL(fontsChanged()), this, SLOT(slotFontsChanged()));
	connect(glFonts, SIGNAL(iconsChanged()), this, SLOT(slotIconsChanged()));

    _menu = new QPopupMenu(this);
	_menu->setCheckable(true);

    _padId = _menu->insertItem(tr("Show scratch pad"), this, SLOT(slotPadClicked()));
	_menu->insertItem(tr("Don't quiz this skill for this item"), this, SLOT(slotDontQuizClicked()));
    __menu->setPopup(_menu);

	updatePad();
}

BoxTrain::~BoxTrain()
{
}

void BoxTrain::slotDontQuizClicked()
{
	_item->removeSkill();
	nextItem();
}

void BoxTrain::slotPadClicked()
{
	_menu->setItemChecked(_padId, !(_menu->isItemChecked(_padId)));
	updatePad();
}

void BoxTrain::updatePad()
{
	if(!(_menu->isItemChecked(_padId)))
	{
		__scribbleH->hide();
		__scribbleV->hide();
		return;
	}

	if(qApp->desktop()->width() < qApp->desktop()->height())
	{
		__scribbleV->clear();
		__scribbleH->hide();
		__scribbleV->show();
	}
	else
	{
		__scribbleH->clear();
		__scribbleV->hide();
		__scribbleH->show();
	}

}

void BoxTrain::resizeEvent(QResizeEvent *e)
{

	updatePad();

	QWidget::resizeEvent(e);

}



void BoxTrain::slotFontsChanged()
{
	__view->setFont(glFonts->font(Fonts::FONT_LargeKana));
}

void BoxTrain::slotIconsChanged()
{
	QString str;

	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)	
		str = "small_";
	else
		str = "large_";

	__menu->setPixmap(glFonts->imagePath() + str + "menu.png");	
	__answer->setPixmap(glFonts->imagePath() + str + "answer.png");	
	__clue->setPixmap(glFonts->imagePath() + str + "clue.png");	
	__ignore->setPixmap(glFonts->imagePath() + str + "right.png");	
	__known->setPixmap(glFonts->imagePath() + str + "tick.png");	
	__unknown->setPixmap(glFonts->imagePath() + str + "x.png");	
	__finish->setPixmap(glFonts->imagePath() + str + "stop.png");	


	
}


void BoxTrain::setList(TrainList *list)
{
	// list should probably be locked here

	if(list->count() == 0)
		return;

	_list = list;
	_item = (*list)[0];
	_pos = 0;

	displayItem();

}

void BoxTrain::nextItem()
{
	if(_pos == (_list->count() -1))
	{
		emit showTrainSelect();
		return;
	}

	++_pos;

	_item = (*_list)[_pos];

	displayItem();
	
}

QString BoxTrain::inTag(const QString& tag, const QString& str)
{
	return "<" + tag + ">" + str + "</" + tag + ">";
}

void BoxTrain::displayItem()
{
	QString txt;

	__view->setText("<p>");
	__clue->setEnabled(true);
	__answer->setEnabled(true);

	if(_item->itemClass() == Item::KanjiItem)
		_kind=tr("Kanji item");
	else
		_kind=tr("Word item");


	if(_item->skill() == Box::KanjiToKana)
	{
		setInstruction("kanji", "reading");
		__view->setText(__view->text() + _item->kanji());
	}
  	else if(_item->skill() == Box::KanjiToMeaning)
	{
		setInstruction("kanji", "meaning");
		__view->setText(__view->text() + _item->kanji());
	}
	else if(_item->skill() == Box::KanaToKanji)
	{
		setInstruction("reading", "kanji");
		__view->setText(__view->text() + _item->kana());
	}
	else if(_item->skill() == Box::KanaToMeaning)
	{
		setInstruction("reading", "meaning");
		__view->setText(__view->text() + QString(_item->kana()));
	}
	else if(_item->skill() == Box::MeaningToKanji)
	{
		setInstruction("meaning", "kanji");
		__view->setText(QString(__view->text() + _item->english()));
	}
	else if(_item->skill() == Box::MeaningToKana)
	{
		setInstruction("meaning", "reading");
		__view->setText(QString(__view->text() + _item->english()));
	}

	__view->setText(__view->text() + "</p>");

	__status->setText(QString("Question %1/%2 Rank=%3").arg(_pos + 1).arg(_list->count()).arg(_item->rank()));
	

}

void BoxTrain::slotFinishClicked()
{
	for(int i=0; i<_list->count(); i++)
		(*_list)[i]->ownerList()->setLocked(false);

	emit showTrainSelect();
}

void BoxTrain::slotUnknownClicked()
{
	_item->demote();
	nextItem();
}

void BoxTrain::slotKnownClicked()
{
	_item->promote();
	nextItem();
}

void BoxTrain::slotClueClicked()
{
	if(_item->skill() == Box::KanaToMeaning || _item->skill() == Box::MeaningToKana)
		__view->setText("<p>Kanji clue: " + QString(_item->kanji())+"</p>\n" + __view->text());
	else if(_item->skill() == Box::KanjiToMeaning || _item->skill() == Box::MeaningToKanji)
		__view->setText("<p>Reading clue: " + QString(_item->kana())+"</p>\n" + __view->text());
	else if(_item->skill() == Box::KanjiToKana || _item->skill() == Box::KanaToKanji)
		__view->setText("<p>Meaning clue: " + QString(_item->english())+"</p>\n" + __view->text());

	__clue->setEnabled(false);

}

void BoxTrain::slotAnswerClicked()
{
	if(_item->skill() == Box::KanaToKanji || _item->skill() == Box::MeaningToKanji)
		__view->setText("<p>Answer: " + QString(_item->kanji())+"</p>" + __view->text());
	else if(_item->skill() == Box::KanjiToKana || _item->skill() == Box::MeaningToKana)
		__view->setText("<p>Answer: " + QString(_item->kana())+"</p>\n" + __view->text());
	else if(_item->skill() == Box::KanjiToMeaning || _item->skill() == Box::KanaToMeaning)
		__view->setText("<p>Answer: " + QString(_item->english())+"</p>\n" + __view->text());

	__answer->setEnabled(false);
}


void BoxTrain::slotIgnoreClicked()
{
	nextItem();
}

void BoxTrain::keyReleaseEvent(QKeyEvent *e)
{
    switch(e->key())
    {
    case Key_Up:
        __known->animateClick();
        break;
    case Key_Down:
        __unknown->animateClick();
        break;
    case Key_Left:
        __clue->animateClick();
        break;
    case Key_Right:
        __ignore->animateClick();
        break;
    case Key_Space:
        __answer->animateClick();
        break;
    }
}

void BoxTrain::setInstruction(const QString& from, const QString& to)
{
	__instruction->setText(tr("%1: from %2 to %3\n").arg(_kind).arg(from).arg(to));

}



