/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qmessagebox.h>
#include "buttongrid.h"
#include "fonts.h"

ButtonGrid::ButtonGrid(QWidget *parent, const char *name)
        : QScrollView(parent, name)
{
    setHScrollBarMode(AlwaysOff);
    setVScrollBarMode(AlwaysOn);
    _buttons.setAutoDelete(true);
    _toggle = false;
}


ButtonGrid::~ButtonGrid()
{
}


void ButtonGrid::resizeEvent(QResizeEvent *e)
{
    QScrollView::resizeEvent(e);
    updateButtons();
}

void ButtonGrid::updateButtons()
{
    if(_buttons.count() == 0)
        return;

    KanjiButton *b;
    int i = 0;

    int lines;
    int cols;
    int bw = _buttons.at(0)->width();
    int bh = _buttons.at(0)->height();

    cols = visibleWidth() / bw;
    lines = _buttons.count() / cols;
    if(_buttons.count() % cols)
        lines++;

    resizeContents(visibleWidth(), lines * bh);

    for(int line = 0; line < lines; line++)
    {
        for(int col = 0; (col < cols) && (i < _buttons.count()); col++,i++)
        {
            b = _buttons.at(i);

            if(_pressed.contains(b->text()[0]))
                b->setOn(true);
            else
                b->setOn(false);
            moveChild(b, col * bw, line * bh);
            b->show();

        }
    }
}

void ButtonGrid::updateFonts()
{

	for(int i=0; i<_buttons.count(); i++)
		_buttons.at(i)->setFont(glFonts->font(Fonts::FONT_Kana));
}


void ButtonGrid::setButtons(const QString& str)
{
    _buttons.clear();
    KanjiButton *b;

    for(int i=0; i < str.length(); i++)
    {
        b = new KanjiButton(QString(str[i]), this);
        b->resize(b->sizeHint());
        if(_toggle)
            b->setToggleButton(true);
        addChild(b, 0, 0);
        connect(b, SIGNAL(kanjiHeld(QChar )), this, SLOT(slotButtonHeld(QChar )));
        connect(b, SIGNAL(kanjiClicked(QChar, QButton::ToggleState)), this, SLOT(slotButtonClicked(QChar, QButton::ToggleState)));
        _buttons.append(b);
        connect(b, SIGNAL(kanjiClicked(QChar)), this, SIGNAL(kanjiSelected(QChar)));
    }

    updateButtons();
}

void ButtonGrid::slotButtonHeld(QChar ch)
{
    emit kanjiHeld(ch);
}

void ButtonGrid::slotButtonClicked(QChar ch, QButton::ToggleState state)
{
    KanjiButton *b;

    if(state == QButton::On)
        _pressed.append(ch);
    else
        _pressed.remove(_pressed.find(ch),1);

    emit(selectionChanged());

}


void ButtonGrid::unselect(QChar ch)
{
    _pressed.remove(_pressed.find(ch),1);
    updateButtons();
}

QString ButtonGrid::selected()
{
    return _pressed;
}

void ButtonGrid::setToggleButtons(bool toggle)
{
    _pressed = "";
    _toggle = toggle;
    updateButtons();
}

void ButtonGrid::setSelected(const QString& str)
{
    _pressed = str;
    updateButtons();
}

QString ButtonGrid::buttons()
{
    QString ret;
    for(int i=0; i<_buttons.count(); i++)
        ret += _buttons.at(i)->text();

    return ret;
}
