/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Based on the Qt "Scribble" example

#include <iostream>
#include <qpainter.h>
#include "canvas.h"

using namespace std;

Canvas::Canvas( QWidget *parent, const char *name)
        : QWidget( parent, name, WNorthWestGravity ), _polyline(2), _strokes(MAX_STROKES),
        _mousePressed( FALSE ), _buffer(width(), height() )
{
    setSizePolicy(QSizePolicy( QSizePolicy::Expanding,QSizePolicy::Expanding));
    _buffer.fill( colorGroup().base() );
    setBackgroundColor( colorGroup().base() );
    #ifndef QWS
    setPenWidth(4);
    #else
    setPenWidth(2);
    #endif
    setPenColor(colorGroup().text());
    _n=0;
    _strokeCount = 0;
    _ps=MAX_STROKES;
}

Canvas::~Canvas()
{
}

bool Canvas::isDrawing()
{
    return _mousePressed;
}


void Canvas::slotClear()
{
    _buffer.fill(colorGroup().base());
    _n=0;
    _strokeCount=0;
    emit(strokesChanged(_strokeCount));
    repaint(FALSE);
}

void Canvas::mousePressEvent(QMouseEvent *e)
{
	if(_strokeCount == 30)
		return;
    _mousePressed = TRUE;
    _polyline[1] =  _polyline[0] = e->pos();
    if (_n < _ps && e->pos().x() >= 0 && e->pos().y() >= 0)
        _strokes[_n++]=_polyline[0];
}

void Canvas::mouseReleaseEvent(QMouseEvent *)
{
	if(_strokeCount == 30)
		return;

    if (_n < _ps)
    {
        _strokes[_n++]=QPoint(-1,-1);
        emit(strokesChanged(++_strokeCount));
    }
    _mousePressed = FALSE;
}

void Canvas::mouseMoveEvent(QMouseEvent *e)
{
	if(_strokeCount == 30)
		return;

    if(e->pos().x() < 0 || e->pos().y() < 0)
        return;

    if(_mousePressed)
    {
        QPainter painter;
        painter.begin(&_buffer);
        painter.setPen(_pen);
        _polyline[1]=_polyline[0];
        _polyline[0] = e->pos();
        painter.drawLine(_polyline[1],_polyline[0]);
        painter.end();

        QRect r = _polyline.boundingRect();
        r = r.normalize();
        r.setLeft( r.left() - penWidth() );
        r.setTop( r.top() - penWidth() );
        r.setRight( r.right() + penWidth() );
        r.setBottom( r.bottom() + penWidth() );
        r=r&visibleRect();
        bitBlt( this, r.x(), r.y(), &_buffer, r.x(), r.y(), r.width(), r.height(),Qt::CopyROP,false );

        if (_n < _ps)
            _strokes[_n++]=_polyline[0];
    }
}

void Canvas::resizeEvent(QResizeEvent *e)
{
    QWidget::resizeEvent(e);

    int w = width() > _buffer.width() ?
            width() : _buffer.width();
    int h = height() > _buffer.height() ?
            height() : _buffer.height();

    QPixmap tmp(_buffer);
    _buffer.resize(w, h);
    _buffer.fill(colorGroup().base());
    bitBlt( &_buffer, 0, 0, &tmp, 0, 0, tmp.width(), tmp.height());
}


void Canvas::paintEvent( QPaintEvent *e )
{
    bitBlt(this, 0, 0, &_buffer, 0, 0, _buffer.width(), _buffer.height(), Qt::CopyROP);
}

void Canvas::slotUndo()
{
    if(_strokeCount == 0)
        return;

    int pos = _n - 2;

    while(_strokes[pos].x() != -1 && pos > 0)
        pos--;

    if(pos != 0)
        _n = pos+1;
    else
        _n = 0;

    emit(strokesChanged(--_strokeCount));

    QPainter painter;
    painter.begin(&_buffer);
    painter.setPen(_pen);
    _buffer.fill(colorGroup().base());

    if(_n != 0)
    {
        for(int i = _n-1; i != 0; i--)
        {
            if((_strokes[i].x() != -1) && (_strokes[i-1].x() != -1))
                painter.drawLine(_strokes[i], _strokes[i-1]);
        }
    }

    painter.end();

    repaint();
}
