/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qapplication.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include "listmanager.h"
#include "editlists.h"
#include "itemlistview.h"
#include "listmanager.h"
#include "stringdialog.h"
#include "fonts.h"
#include "listsettingsdialog.h"
#include "itemsettingsdialog.h"

using std::cerr;

EditLists::EditLists(QWidget *parent, const char *name, WFlags f)
        : EditListsWidget(parent, name, f)
{
    connect(__listsCB, SIGNAL(activated(int)), this, SLOT(slotCBActivated(int)));
    connect(glListManager, SIGNAL(doneLoading()), this, SLOT(slotListsLoaded()));

	connect(__itemDeleteButton, SIGNAL(clicked()), this, SLOT(slotItemDeleteClicked()));
	connect(__itemDetailsButton, SIGNAL(clicked()), this, SLOT(slotItemDetailsClicked()));
	connect(__itemSettingsButton, SIGNAL(clicked()), this, SLOT(slotItemSettingsClicked()));

	connect(glFonts, SIGNAL(fontsChanged()), this, SLOT(slotFontsChanged()));
	connect(glFonts, SIGNAL(iconsChanged()), this, SLOT(slotIconsChanged()));

    _listMenu = new QPopupMenu(this);

    _listMenu->insertItem("Create new list...", this, SLOT(slotNewClicked()));

    __listMenu->setPopup(_listMenu);

    _currList = 0;
	_removeListId = -1;


}

EditLists::~EditLists()
{
}

void EditLists::slotFontsChanged()
{
	__list->updateFonts();
}

void EditLists::slotIconsChanged()
{
	QString str;

	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)	
		str = "small_";
	else
		str = "large_";

	__listMenu->setPixmap(glFonts->imagePath() + str + "menu.png");	
	__itemDeleteButton->setPixmap(glFonts->imagePath() + str + "delete.png");	
	__itemDetailsButton->setPixmap(glFonts->imagePath() + str + "details.png");	
	__itemSettingsButton->setPixmap(glFonts->imagePath() + str + "editquizsettings.png");	

}	

void EditLists::slotItemDeleteClicked()
{
	if((_currList == 0) || (__list->currIndex() == -1))
		return;

	if(_currList->locked())
	{
		QMessageBox::information(this, "JARDS Mobile", tr("Items in this list can't be deleted.\nThe list is used in a quiz."),1);
		return;
	}

    _currList->remove(__list->currIndex());
    __list->remove(__list->currIndex());

	
}

void EditLists::slotItemDetailsClicked()
{
	if(__list->currIndex() == -1)
		return;

	emit showItem(__list->currItem());
}

void EditLists::slotItemSettingsClicked()
{
	if(__list->currIndex() == -1)
		return;

	ItemSettingsDialog *dia = new ItemSettingsDialog(this);
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	dia->setItem(__list->currItem(), _currList->defaultSkills());
	if(dia->exec())
		_currList->setChanged(true);
	delete dia;
}

void EditLists::slotDeleteClicked()
{
    if(_currList == 0)
        return;

	if(_currList->locked())
	{
		QMessageBox::information(this, "JARDS Mobile", tr("This list can't be deleted.\nIt is used in a quiz."),1);
		return;
	}

    if( QMessageBox::warning( this, "JARDS Mobile", tr("Are you sure you want to delete \"%1\" ?")
                                                .arg(_currList->name()),
                                  QMessageBox::Ok | QMessageBox::Default,
                                  QMessageBox::Cancel | QMessageBox::Escape )
    == QMessageBox::Cancel)
        return;

    glListManager->deleteList(_currList);

	if(glListManager->count() == 0)
	{
		_listMenu->removeItem(_removeListId);
		_listMenu->removeItem(_saveAllId);
		_listMenu->removeItem(_listSettingsId);
		_saveAllId = -1;
		_removeListId = -1;
		_listSettingsId = -1;
	}

    reset();

}

void EditLists::slotCBActivated(int index)
{
	if((index == -1) || (glListManager->count() == 0))
		return;

	glListManager->setCurrentList(index);

    _currList = glListManager->currentList();

	__list->setList(_currList);
	
}

void EditLists::slotListsLoaded()
{
	if(glListManager->count() == 0)
		return;

    _currList = glListManager->listAt(0);

	__list->setList(_currList);

	addMenuItems();
	
}

void EditLists::addMenuItems()
{

   	_saveAllId = _listMenu->insertItem(tr("Save all lists..."), this, SLOT(slotSaveClicked()));
   _removeListId = _listMenu->insertItem(tr("Delete this list..."), this, SLOT(slotDeleteClicked()));
   _listSettingsId = _listMenu->insertItem(tr("List settings..."), this, SLOT(slotListSettingsClicked()));
}

void EditLists::slotListSettingsClicked()
{
	ListSettingsDialog *dia = new ListSettingsDialog(this);
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	dia->setList(_currList);
	dia->exec();
}

void EditLists::slotSaveClicked()
{
    glListManager->saveAll();
}

void EditLists::slotExportClicked()
{

}

void EditLists::slotNewClicked()
{
    bool succ = false;
    QString name;

    StringDialog d(this);

    do
    {
        name = d.getString("Name");
        if(name.length() == 0)
			return;
        else if (glListManager->nameExists(name))
            QMessageBox::information(0, "JARDS Mobile", tr("A list with that name already exists.\nPlease change the name."));
        else
            succ = true;
    } while(!succ);

    _currList = glListManager->appendList(name);

    updateComboBox();

    __listsCB->setCurrentItem(__listsCB->count() - 1);

	glListManager->setCurrentList(__listsCB->count() - 1);

    __list->clear();

    __list->setList(_currList);

	if(glListManager->count() == 1)
	{
		addMenuItems();
	}

}

void EditLists::updateComboBox()
{
    __listsCB->clear();

   QStringList list = glListManager->listNames();

   for(int i=0; i<list.count(); i++)
        __listsCB->insertItem(list[i]);

}

void EditLists::reset()
{
    updateComboBox();

    __list->clear();

    if(glListManager->count() == 0)
    {
        _currList = 0;
        return;
    }

    _currList = glListManager->listAt(0);

    __listsCB->setCurrentItem(0);

    if(glListManager->listAt(0)->count() == 0)
        return;

    __list->setList(_currList);
}
