/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include "fontdialog.h"
#ifdef USE_QPE
  #include "fontselectordialog.h"
#else
  #include <qfontdialog.h>
#endif

using std::cerr;

FontDialog::FontDialog(QWidget *parent, const char *name)
 : FontDialogBase(parent, name, true, 0)
{
	connect(__changeGeneric, SIGNAL(clicked()), this, SLOT(slotChangeGeneric()));
	connect(__changeKana, SIGNAL(clicked()), this, SLOT(slotChangeKana()));
	connect(__changeLargeKanji, SIGNAL(clicked()), this, SLOT(slotChangeLargeKanji()));
	connect(__changeLargeKana, SIGNAL(clicked()), this, SLOT(slotChangeLargeKana()));

	_generic = glFonts->font(Fonts::FONT_Generic);
	_kana = glFonts->font(Fonts::FONT_Kana);
	_largeKana = glFonts->font(Fonts::FONT_LargeKana);
	_largeKanji = glFonts->font(Fonts::FONT_LargeKanji);
	updateLabels();
}	


FontDialog::~FontDialog()
{
}

void FontDialog::updateLabels()
{

	__genericName->setText(_generic.family() + QString(",%1 ").arg(_generic.pointSize()));
//	__genericName->setFont(_fonts.font(Fonts::FONT_Generic));
	__kanaName->setText(_kana.family() + QString(",%1 ").arg(_kana.pointSize()));
//	__kanaName->setFont(_fonts.font(Fonts::FONT_Kana));
	__largeKanjiName->setText(_largeKanji.family() + QString(",%1 ").arg(_largeKanji.pointSize()));
//	__largeKanjiName->setFont(_fonts.font(Fonts::FONT_LargeKanji));
	__largeKanaName->setText(_largeKana.family() + QString(",%1 ").arg(_largeKana.pointSize()));
//	__largeKanaName->setFont(_fonts.font(Fonts::FONT_LargeKana));
}

void FontDialog::slotChangeGeneric()
{
	changeFont(_generic);
}

void FontDialog::slotChangeKana()
{
	changeFont(_kana);
}

void FontDialog::slotChangeLargeKanji()
{
	changeFont(_largeKanji);
}

void FontDialog::slotChangeLargeKana()
{
	changeFont(_largeKana);
}

void FontDialog::changeFont(QFont& font)
{
#ifdef USE_QPE

 	FontSelectorDialog *dia = new FontSelectorDialog();
	dia->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));

	dia->setSelectedFont(font);

	if(dia->exec())
	{
		font = dia->selectedFont();
		updateLabels();
	}

	delete dia;
#else
	bool ok;
	QFont f = QFontDialog::getFont(&ok, QFont(), this);
	if(ok)
	{
		font = f;
		updateLabels();
	}
#endif
}

void FontDialog::apply()
{
	glFonts->setFont(Fonts::FONT_Generic, _generic);
	glFonts->setFont(Fonts::FONT_Kana, _kana);
	glFonts->setFont(Fonts::FONT_LargeKana, _largeKana);
	glFonts->setFont(Fonts::FONT_LargeKanji, _largeKanji);
}

