/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qapplication.h>
#include "configmanager.h"
#include "fonts.h"


Fonts::Fonts()
{
	_fontsChanged = false;
	_iconsChanged = false;
}

Fonts::~Fonts()
{
}

void Fonts::setIconSize(IconSizes size)
{
	_iconSize = size;
	_iconsChanged = true;
}

void Fonts::emitSignals()
{
	if(_fontsChanged)
	{
		emit fontsChanged();
		_fontsChanged = false;
	}
	if(_iconsChanged)
	{
		emit iconsChanged();
		_iconsChanged = false;
	}
}

void Fonts::saveSettings()
{
	QFont *f;

	glConfigManager->setGroup("Fonts");

	f = &_fonts[FONT_Generic];
	glConfigManager->writeEntry("Generic", QString("%1,%2,%3,%4").arg(f->family()).arg(f->pointSize()).arg(f->weight()).arg(f->italic()));
	
	f = &_fonts[FONT_Kana];
	glConfigManager->writeEntry("Kana", QString("%1,%2,%3,%4").arg(f->family()).arg(f->pointSize()).arg(f->weight()).arg(f->italic()));

	f = &_fonts[FONT_LargeKanji];
	glConfigManager->writeEntry("LargeKanji", QString("%1,%2,%3,%4").arg(f->family()).arg(f->pointSize()).arg(f->weight()).arg(f->italic()));

	f = &_fonts[FONT_LargeKana];
	glConfigManager->writeEntry("LargeKana", QString("%1,%2,%3,%4").arg(f->family()).arg(f->pointSize()).arg(f->weight()).arg(f->italic()));

	glConfigManager->setGroup("Icons");
	if(_iconSize == ICONSIZE_Small)
		glConfigManager->writeEntry("Size", "small");
	else
		glConfigManager->writeEntry("Size", "large");

}

void Fonts::restoreSettings()
{
	QString str;
	QStringList list;
	glConfigManager->setGroup("Fonts");

	list = QStringList::split(",", glConfigManager->readEntry("Generic"));
	if(list.count() == 0)
		_fonts[FONT_Generic] = qApp->font();
	else
		_fonts[FONT_Generic] = QFont(list[0], list[1].toInt(), list[2].toInt(), list[3].toInt());

	list = QStringList::split(",", glConfigManager->readEntry("Kana"));
	if(list.count() == 0)
		_fonts[FONT_Kana] = qApp->font();
	else
		_fonts[FONT_Kana] = QFont(list[0], list[1].toInt(), list[2].toInt(), list[3].toInt());

	list = QStringList::split(",", glConfigManager->readEntry("LargeKanji"));
	if(list.count() == 0)
		_fonts[FONT_LargeKanji] = qApp->font();
	else
		_fonts[FONT_LargeKanji] = QFont(list[0], list[1].toInt(), list[2].toInt(), list[3].toInt());
	
	list = QStringList::split(",", glConfigManager->readEntry("LargeKana"));
	if(list.count() == 0)
		_fonts[FONT_LargeKana] = qApp->font();
	else
		_fonts[FONT_LargeKana] = QFont(list[0], list[1].toInt(), list[2].toInt(), list[3].toInt());

	glConfigManager->setGroup("Icons");
	str = glConfigManager->readEntry("Size");

	if(str == "small")
		_iconSize = ICONSIZE_Small;
	else
		_iconSize = ICONSIZE_Large;
	

	_fontsChanged = true;
	_iconsChanged = true;

}

Fonts *glFonts = 0;
