/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef FONTS_H
#define FONTS_H

#include <qobject.h>
#include <qfont.h>

class Fonts : public QObject
{
	Q_OBJECT
public:

	enum FontRoles {FONT_Generic = 0, FONT_Kana, FONT_LargeKanji, FONT_LargeKana};
	enum IconSizes {ICONSIZE_Small, ICONSIZE_Large};

	Fonts();
	~Fonts();

	QFont font(FontRoles font) const {return _fonts[font];};
	void setFont(FontRoles font, const QFont& source) {_fontsChanged =  true; _fonts[font] = source;};

	void setIconSize(IconSizes size);
	IconSizes iconSize() const {return _iconSize;};

	void emitSignals();

	void saveSettings();
	void restoreSettings();

	QString imagePath() const {return _imageDir;};
	void setImagePath(const QString& str) {_imageDir = str;};

signals:
	void fontsChanged();
	void iconsChanged();

private:
	QFont _fonts[4];
	QString _imageDir;
	IconSizes _iconSize;
	bool _fontsChanged;
	bool _iconsChanged;


};

extern Fonts *glFonts;



#endif