/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <ctype.h>
#include <unistd.h>
#include <qapplication.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include "hcr.h"
#include "canvas.h"
#include "hcrengine.h"

using std::cerr;


HCR::HCR(QWidget *parent, const char *name, WFlags f)
        : HCRWidget(parent, name, f)
{
    dummy1->hide();
    dummy2->hide();

    _timer = new QTimer(this);

    _kanjiButtons.setAutoDelete(true);
    slotStrokesChanged(0);

    connect(canvas, SIGNAL(strokesChanged(int)), this, SLOT(slotStrokesChanged(int)));
    connect(undoB, SIGNAL(clicked()), canvas, SLOT(slotUndo()));
    connect(clearB, SIGNAL(clicked()), canvas, SLOT(slotClear()));
    connect(clearB, SIGNAL(clicked()), this, SLOT(slotClear()));
    connect(_timer, SIGNAL(timeout()), this, SLOT(slotLookup()));

	connect(glFonts, SIGNAL(iconsChanged()), this, SLOT(slotIconsChanged()));
}

HCR::~HCR()
{
}

void HCR::slotIconsChanged()
{
	QString str;

	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)	
		str = "small_";
	else
		str = "large_";

	clearB->setPixmap(glFonts->imagePath() + str + "clear.png");	
	undoB->setPixmap(glFonts->imagePath() + str + "undo.png");	
}


void HCR::slotClear()
{
    _kanjiButtons.clear();
}

bool HCR::initialize(const QString& kpEngineFileName, const QString& strokeDataFileName)
{
    if(_engine == 0)
        _engine = new HCREngine();

    return _engine->initialize(kpEngineFileName, strokeDataFileName);

}

void HCR::slotLookup()
{
	if(canvas->isDrawing())
	{
        _timer->start(400, true);		// try again later
		return;
	}
    if(_strokes == 0)
        return;

    _engine->lookup(this, canvas->strokes(), canvas->numPoints());

}

void HCR::slotLookupResults(QString str)
{
    if(str.isEmpty())
        return;
        
    KanjiButton *kb;
   
    _kanjiButtons.clear();

    for(int i=0; i<str.length(); i++)
    {
		if(i>3)
			return;
        kb = new KanjiButton(candidatesFrame);
        kb->setText(QString(str[i]));
        kb->show();
        _kanjiButtons.append(kb);
        candidatesLayout->add(kb);
        connect(kb, SIGNAL(kanjiClicked(QChar)), this, SLOT(slotKanjiSelected(QChar)));
        connect(kb, SIGNAL(kanjiHeld(QChar)), this, SLOT(slotKanjiHeld(QChar)));
    }
}

void HCR::slotKanjiSelected(QChar ch)
{
    emit(kanjiSelected(ch));
}

void HCR::slotKanjiHeld(QChar ch)
{
    emit(kanjiHeld(ch));
}

void HCR::slotStrokesChanged(int n)
{

    strokesL->setText(QString(tr("%1 strokes")).arg(n));

    if(n == 0)
    {
        _kanjiButtons.clear();
    }

    _strokes = n;

    if(n != 0)
        _timer->start(400, true);

}



HCREngine *HCR::_engine = 0;
