/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <ctype.h>
#include <unistd.h>
#include <qapplication.h>
#include <qfile.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include "hcr.h"
#include "canvas.h"
#include "hcrengine.h"
#include "item.h"

using std::cerr;


HCREngine::HCREngine(QObject *parent, const char *name)
        : QObject(parent, name)
{
    _initialized = false;
}

HCREngine::~HCREngine()
{}

bool HCREngine::initialize(const QString& kpEngineFileName, const QString& strokeDataFileName)
{
    int in_fd[2];
    int out_fd[2];

    if(pipe(in_fd))
    {
        cerr << "can't create pipe" << endl;
        return false;
    }

    if(pipe(out_fd))
    {
        cerr << "can't create pipe" << endl;
        return false;
    }

    if(!QFile(strokeDataFileName).exists())
    {
        cerr << "can't find strokedata" << endl;
        return false;
    }


    _enginePID = fork ();
    if(_enginePID == 0)
    { /* child */
        ::close (in_fd[1]);
        ::close (out_fd[0]);

        dup2 (in_fd[0], STDIN_FILENO);
        dup2 (out_fd[1], STDOUT_FILENO);
        execlp(kpEngineFileName,kpEngineFileName,"--data-file", QString(strokeDataFileName).latin1(), NULL);

    }
    else if(_enginePID < 0)
    {  /* failure */
        cerr << "spawn failed" << endl;
        return false;
    }
    /* Parent process */

    ::close (in_fd[0]);
    ::close (out_fd[1]);

    _toEngine = fdopen(in_fd[1], "w");
    _fromEngine = fdopen(out_fd[0], "r");

    _sn = new QSocketNotifier(out_fd[0], QSocketNotifier::Read, this);
    connect(_sn, SIGNAL(activated(int)), SLOT(slotGetdata(int)));
    _initialized = true;
    return true;
}

void HCREngine::slotGetdata(int k)
{
    QString hits;
    static char buffer[BUFLEN];
    static char *p;
    int i;
    int num_guesses = 0;

    if (!fgets(buffer, BUFLEN, _fromEngine))
    {
        _sn->disconnect();
        cerr << "kpengine no longer exists" << endl;    // Try init again... !
    }

    if(buffer[0] == 'K')
    {
        uint t1;
        p = buffer+1;
        for (i=0; i<MAX_GUESSES; i++)
        {
            while (*p && isspace(*p)) p++;
            if (!*p || sscanf(p, "%4x", &t1) != 1)
            {
                i--;
                break;
            }

            hits += glKanjiDic->getKanjiByJIS((ushort)t1).kanji()[0];

            while (*p && !isspace(*p))
                p++;
        }
        num_guesses = i+1;

    }

    emit lookupResults(hits);
}

bool HCREngine::lookup(HCR *caller, QPointArray strokes, int numPoints)
{
    if(!_initialized)
        return false;

    int i;

    disconnect(this, SIGNAL(lookupResults(QString)),0,0);
    connect(this, SIGNAL(lookupResults(QString)), caller, SLOT(slotLookupResults(QString)));

    for (i=0; i<numPoints; i++)
    {
        if (!((strokes[i].x()==-1)&&(strokes[i].y()==-1)))
            fprintf(_toEngine, "%d %d ",strokes[i].x(),strokes[i].y());
        else
            fprintf(_toEngine, "\n");
    }
    fprintf(_toEngine, "\n");
    fflush(_toEngine);

    return true;

}

