/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qapplication.h>
#include "horizbuttons.h"

HorizButtons::HorizButtons(QWidget *parent, const char *name)
        : HorizButtonsWidget(parent, name)
{
    scrollV->setVScrollBarMode(QScrollView::AlwaysOff);
    scrollV->setHScrollBarMode(QScrollView::AlwaysOff);

    _hBox = new QHBox(scrollV->viewport());
    scrollV->addChild(_hBox);

    _buttons.setAutoDelete(true);

    connect(leftB, SIGNAL(clicked()), this, SLOT(slotLeftClicked()));
    connect(rightB, SIGNAL(clicked()), this, SLOT(slotRightClicked()));
}



HorizButtons::~HorizButtons()
{
}

void HorizButtons::slotLeftClicked()
{
    scrollV->scrollBy(-scrollV->width() / 5, 0);
    adjustButtons();
}

void HorizButtons::slotRightClicked()
{
    scrollV->scrollBy(scrollV->width() / 5, 0);
    adjustButtons();
}

void HorizButtons::adjustButtons()
{
    qApp->processEvents();

    if(scrollV->contentsX() != 0)
        leftB->setEnabled(true);
    else
        leftB->setEnabled(false);

    if( _hBox->width() - scrollV->contentsX() > scrollV->width())
        rightB->setEnabled(true);
    else
        rightB->setEnabled(false);
}

void HorizButtons::resizeEvent(QResizeEvent *e)
{
    adjustButtons();
}

void HorizButtons::setButtons(QString labels)
{
    // Needs to go faster! remove hboxlayout and just put'em in a row

    KanjiButton *b;

    _buttons.clear();

    for(int i=0; i < labels.length(); i++)
    {
        b = new KanjiButton(QString(labels[i]), _hBox);
        connect(b, SIGNAL(kanjiClicked(QChar)), this, SIGNAL(buttonClicked(QChar)));
        connect(b, SIGNAL(kanjiHeld(QChar)), this, SIGNAL(buttonHeld(QChar)));
        connect(b, SIGNAL(kanjiClicked(QChar, QButton::ToggleState)), this, SIGNAL(buttonClicked(QChar, QButton::ToggleState)));
        b->resize(b->sizeHint());
        b->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
        _buttons.append(b);
        b->show();
    }

    if(labels.length() != 0)
        setMinimumSize(0, b->height());

    adjustButtons();
    
    updateGeometry();
}

