/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <iostream>
#include <qregexp.h>
#include "hypertextbrowser.h"
#include "fonts.h"

using std::cerr;

HypertextBrowser::HypertextBrowser(QWidget *parent, const char *name) : QTextBrowser(parent, name)
{
    setLinkUnderline(false);
    setPaper(white);
    setFrameStyle(NoFrame);
    setTextFormat(RichText);
    _markedColor = "#bb0000";
    _pos = -1;

	_largeKana = new QStyleSheetItem(&_sheet, "largekana");
	_largeKanji = new QStyleSheetItem(&_sheet, "largekanji");
	_kana = new QStyleSheetItem(&_sheet, "kana");
	_generic = new QStyleSheetItem(&_sheet, "generic");

	updateFonts();
}

HypertextBrowser::~HypertextBrowser()
{}

void HypertextBrowser::updateFonts()
{
	_largeKana->setFontSize(glFonts->font(Fonts::FONT_LargeKana).pointSize());
	_largeKana->setFontFamily(glFonts->font(Fonts::FONT_LargeKana).family());
	_generic->setFontSize(glFonts->font(Fonts::FONT_Generic).pointSize());
	_generic->setFontFamily(glFonts->font(Fonts::FONT_Generic).family());
	_kana->setFontSize(glFonts->font(Fonts::FONT_Kana).pointSize());
	_kana->setFontFamily(glFonts->font(Fonts::FONT_Kana).family());
	_largeKanji->setFontSize(glFonts->font(Fonts::FONT_LargeKanji).pointSize());
	_largeKanji->setFontFamily(glFonts->font(Fonts::FONT_LargeKanji).family());

	setStyleSheet(&_sheet);

	setText("<qt>"+text()+"</qt>");

}

void HypertextBrowser::setSource(const QString& name)
{
    emit linkClicked(name, true);
}


void HypertextBrowser::select()
{
    if(_pos == -1)
        return;
    int start = text().find('=', _pos)+1;
    int end = text().find('>', start);
    emit linkClicked(text().mid(start, end-start), false);
}

void HypertextBrowser::right()
{
    QString str = removeHilight();

    if(_pos == -1)
        _pos = str.find("<a href");
    else
        _pos = str.find("<a href", _pos+1);

    if(_pos == -1)
        _pos = str.find("<a href");

    insertHilight(str);
}

void HypertextBrowser::left()
{
    QString str = removeHilight();

    if(_pos == -1)
        _pos = str.findRev("<a href");
    else
        _pos = str.findRev("<a href", _pos-1);

    if(_pos == -1)
        _pos = str.findRev("<a href");

    insertHilight(str);
}

void HypertextBrowser::setText(const QString& str)
{
    _pos = -1;
    QTextBrowser::setText(str);
}

void HypertextBrowser::resetCurrentLink()
{
    _pos = -1;

    QTextBrowser::setText(removeHilight());
}

QString HypertextBrowser::removeHilight()
{
    QString str = text();

    str.replace(QRegExp(QString("<font color=%1>").arg(_markedColor)), "");

    return str;
}

void HypertextBrowser::insertHilight(QString& str)
{

    str.insert(str.find('>', _pos)+1,QString("<font color=%1>").arg(_markedColor));

    QTextBrowser::setText(str);
}

void HypertextBrowser::keyPressEvent(QKeyEvent *e)
{
    if(e->key() == Key_Space)
        select();

    else if(e->key() == Key_Right)
        right();

    else if(e->key() == Key_Left)
        left();
}
