/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include "itemsheet.h"
#include "itemdetails.h"
#include "fonts.h"

ItemDetails::ItemDetails(QWidget *parent, const char *name)
 : ItemDetailsWidget(parent, name)
{
    _pop = new ItemPopup(this);
    __menu->setPopup(_pop);
    _history.setAutoDelete(true);

    connect(__history, SIGNAL(activated(int)), this,  SLOT(slotHistoryActivated(int)));
	connect(glFonts, SIGNAL(fontsChanged()), this, SLOT(slotFontsChanged()));
	connect(glFonts, SIGNAL(iconsChanged()), this, SLOT(slotIconsChanged()));
    _maxHistory = 10;

}


ItemDetails::~ItemDetails()
{
}

void ItemDetails::slotIconsChanged()
{
	QString str;

	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)	
		str = "small_";
	else
		str = "large_";

	__menu->setPixmap(glFonts->imagePath() + str + "menu.png");	
}

void ItemDetails::slotFontsChanged()
{
	__history->setFont(glFonts->font(Fonts::FONT_Kana));
	__sheet->updateFonts();
}

void ItemDetails::setupGui(const Item *item)
{
    _pop->setItem(item, ItemPopup::ToList);

}


void ItemDetails::slotShowItem(const Item *item)
{

    if(_history.count() == _maxHistory)
    {
        _history.removeFirst();
        __history->removeItem(__history->count()-1);
    }
 
    if(item->itemClass() == Item::KanjiItem)
        __history->insertItem(QString("Kanji: %1").arg(item->kanji()[0]), 0);
    else if(item->itemClass() == Item::WordItem)
        __history->insertItem(QString("Word: %1 [%2]").arg(item->kanji()).arg(item->kana()), 0);

    __sheet->setItem(item);
    setupGui(item);
    _history.append(item->clone());
}

void ItemDetails::slotToSearch()
{
    emit focusSearch(__sheet->item()->kanji()[0]);
}

void ItemDetails::slotHistoryActivated(int index)
{
	if(index == -1 || _history.count() == 0)
		return;

    int x = __history->count() - 1 - index;
    setupGui(_history.at(x));
    __sheet->setItem(_history.at(x));
}

void ItemDetails::slotAddToList()
{
}


