/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qscrollview.h>
#include <qlistbox.h>
#include <qapplication.h>
#include <qcolor.h>
#include <qstring.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qlabel.h>
#include <qevent.h>
#include <qlayout.h>
#include <qregexp.h>
#include "itemlistview.h"
#include "dictmanager.h"
#include "listmanager.h"
#include "itempopup.h"
#include "fonts.h"

using namespace std;

NarauListView::NarauListView(QWidget *parent, const char *name)
        : ItemListViewWidget(parent, name)
{
    __item->setHScrollBarMode(QScrollView::AlwaysOff);
    __lv->setHScrollBarMode(QScrollView::AlwaysOff);
    setFocusPolicy(StrongFocus);
    __item->viewport()->setBackgroundColor(white);

    connect(__lv, SIGNAL(highlighted(int)), this, SLOT(slotHighlighted(int)));
    connect(__item, SIGNAL(itemClicked(const Item*, QPoint)), this, SLOT(slotItemClicked(const Item *, QPoint)));

    connect(__item, SIGNAL(wordPartClicked(const Item*, QPoint)), this, SLOT(slotWordPartClicked(const Item *, QPoint)));


    _pop = new ItemPopup(this);

	updateFonts();

//	ItemListViewWidgetLayout->setRowStretch(0, 0);
//	ItemListViewWidgetLayout->setRowStretch(1, 0);
}

NarauListView::~NarauListView()
{
}

void NarauListView::updateFonts()
{
	__lv->setFont(glFonts->font(Fonts::FONT_Kana));
	__item->updateFonts();
}

void NarauListView::keyPressEvent(QKeyEvent *e)
{
    if(__lv->currentItem() == -1)
    {
        __lv->setSelected(0, true);
        slotHighlighted(0);
        return;
    }

    if(e->key() == Key_Space)
        __item->select();

    else if(e->key() == Key_Right)
        __item->right();

    else if(e->key() == Key_Left)
        __item->left();
    else if(e->key() == Key_Down && __lv->currentItem() < count() -1)
    {
        __lv->setSelected(__lv->currentItem()+1, true);
        __lv->ensureCurrentVisible();
        __item->setItem(currItem());
    }
    else if(e->key() == Key_Up && __lv->currentItem() > 0)
    {
        __lv->setSelected(__lv->currentItem()-1, true);
        __lv->ensureCurrentVisible();
		__item->setItem(currItem());
    }
}

void NarauListView::clear()
{
    __lv->clear();
	__item->clear();
}

void NarauListView::slotHighlighted(int index)
{
	if(index == -1)
		return;
    __item->setItem(currItem());
    __lv->ensureCurrentVisible();
}

void NarauListView::slotItemClicked(const Item *item, QPoint pos)
{
	if(modifiable())
    	_pop->setItem(item, ItemPopup::PopupFlags(ItemPopup::ToDetails | ItemPopup::Delete), __lv->currentItem());
	else
    	_pop->setItem(item, ItemPopup::PopupFlags(ItemPopup::ToDetails | ItemPopup::ToList), __lv->currentItem());

    _pop->exec(pos, 0);
}

void NarauListView::slotWordPartClicked(const Item *item, QPoint pos)
{
	if(modifiable())
    	_pop->setItem(item, ItemPopup::PopupFlags(ItemPopup::ToDetails), __lv->currentItem());
	else
   	_pop->setItem(item, ItemPopup::PopupFlags(ItemPopup::ToDetails | ItemPopup::ToList), __lv->currentItem());
    _pop->exec(pos, 0);
}

void NarauListView::resizeEvent(QResizeEvent *ev)
{
	__lv->resize(width(), 2 * (height() / 3));
	__item->resize(width(), height()/3);
	__item->move(0, __lv->frameGeometry().bottom());
}



ItemListView::ItemListView(QWidget *parent, const char *name) : NarauListView(parent, name)
{
//    connect(_pop, SIGNAL(remove(int)), this, SLOT(slotRemove(int)));
    connect(__item, SIGNAL(itemClicked(const Item*, QPoint)), this, SLOT(slotItemClicked(const Item *, QPoint)));
}

ItemListView::~ItemListView()
{
}

void ItemListView::remove(int index)
{
	if(index == __lv->currentItem())
		__item->clear();

//    if(_currItem == 0 && count() == 1)
//        _currItem = -1;
//    else if(_currItem >= index)
//        _currItem--;

    __lv->removeItem(index);

    if(__lv->currentItem() != -1)
        __item->setItem((*_list)[__lv->currentItem()]);
}

void ItemListView::setList(ItemList *list)
{
	_list = list;

	slotUpdateList();

	connect(_list, SIGNAL(itemAdded()), this, SLOT(slotUpdateList()));
	
}

void ItemListView::slotUpdateList()
{
	clear();

	QStringList list;

    for(int i=0; i < count(); i++)
		list.append((*_list)[i]->infoString());

     __lv->insertStringList(list);

}

SearchResultsListView::SearchResultsListView(QWidget *parent, const char *name)
        : NarauListView(parent, name)
{
/*
#ifdef QWS
    __lv->setFont(FontManager::unicodeFont(FontManager::Fixed));
#endif

    __item->setVScrollBarMode(QScrollView::AlwaysOff);
    __item->setHScrollBarMode(QScrollView::AlwaysOff);
    __lv->setHScrollBarMode(QScrollView::AlwaysOff);
    setFocusPolicy(StrongFocus);
    __item->viewport()->setBackgroundColor(white);

    connect(__lv, SIGNAL(highlighted(int)), this, SLOT(slotHighlighted(int)));
    connect(__item, SIGNAL(itemClicked(const Item*, QPoint)), this, SLOT(slotItemClicked(const Item *, QPoint)));

    _pop = new ItemPopup(this);
*/

//    connect(__item, SIGNAL(itemClicked(const Item*, QPoint)), this, SLOT(slotItemClicked(const Item *, QPoint)));

	_word = 0;
	_list = 0;
	
}

SearchResultsListView::~SearchResultsListView()
{
	if(_word)
		delete _word;
	if(_list)
		delete _list;
}

Item *SearchResultsListView::currItem()
{
	if(_word != 0)
		delete _word;

	_word = new Word(_list->at(__lv->currentItem()));

	return _word;
};

void SearchResultsListView::slotUpdateList()
{
	clear();

	QStringList list;

    for(int i=0; i < count(); i++)
		list.append(_list->at(i)->infoString());

     __lv->insertStringList(list);

}

void SearchResultsListView::setList(QList<const DictWord> *list)
{
	if(_list)
		delete _list;

	_list = list;

	slotUpdateList();
}

