/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ITEMLISTVIEW_H
#define ITEMLISTVIEW_H

#include <qscrollview.h>
#include <qpopupmenu.h>
#include <qvbox.h>
#include <qmenudata.h>
#include <qstring.h>
#include <qlistbox.h>
#include "dictionary.h"
#include "itemlistviewwidget.h"
#include "kanjidic.h"
#include "resultsitem.h"
#include "item.h"

class ItemPopup;

class NarauListView : public ItemListViewWidget
{
    Q_OBJECT
public:

    NarauListView(QWidget *parent = 0, const char *name = 0);
    ~NarauListView();

    void clear();
    int currIndex() {return __lv->currentItem();};
	virtual Item *currItem() {return 0;};
	void updateFonts();

signals:
    void showItem(const Item *);
    void addToList(const Item *);

protected slots:
    void slotHighlighted(int index);
    void slotItemClicked(const Item *item, QPoint pos);
    void slotWordPartClicked(const Item *item, QPoint pos);

protected:
	virtual bool modifiable() const = 0;
	virtual int count() const = 0;
    void keyPressEvent(QKeyEvent *e);
    void kanjiClicked(QChar ch);

private:
	void resizeEvent(QResizeEvent *ev);
   ItemPopup *_pop;

};

class ItemListView : public NarauListView
{
Q_OBJECT

public:
    ItemListView(QWidget *parent = 0, const char *name = 0);
    ~ItemListView();

	void setList(ItemList *list);
	void remove(int index);
	Item *currItem() {return (*_list)[__lv->currentItem()];};

private slots:
	void slotUpdateList();

private:
	bool modifiable() const {return true;};
	int count() const {return _list->count();};
    ItemList *_list;
    ItemPopup *_pop;

};

class SearchResultsListView : public NarauListView
{
	Q_OBJECT
public:

    SearchResultsListView(QWidget *parent = 0, const char *name = 0);
    ~SearchResultsListView();

	void setList(QList<const DictWord> *list);
	Item *currItem();

private slots:
	void slotUpdateList();


private:
	bool modifiable() const {return false;};
	int count() const {if(_list == 0) return 0; return _list->count();};
	Word *_word;
	QList<const DictWord> *_list;
    ItemPopup *_pop;

};


#endif
