/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "itempopup.h"
#include "item.h"
#include "fonts.h"
#include <qlabel.h>

ItemPopup::ItemPopup(QWidget *parent, const char *name)
        : QPopupMenu(parent, name)
{
    _item = 0;
    _id = -1;
}


ItemPopup::~ItemPopup()
{
    if(_item)
        delete _item;
}

void ItemPopup::setItem(const Item *item, PopupFlags flags, int id)
{
    if(_item)
        delete _item;

    _id = id;

    _item = item->clone();

    clear();

    if(flags & ToDetails)
    {
        QLabel *label = new QLabel(this);
		if(item->itemClass() == Item::KanjiItem)
			label->setFont(glFonts->font(Fonts::FONT_LargeKanji));
		else
			label->setFont(glFonts->font(Fonts::FONT_LargeKana));
        label->setText(QString("<center>%1</center>").arg(item->kanji()));
        insertItem(label);

        label = new QLabel(this);
		label->setFont(glFonts->font(Fonts::FONT_LargeKana));
        label->setText(QString("<center>%1</center>").arg(item->kana()));
        insertItem(label);

        label = new QLabel(this);
        label->setText("<center>" + item->english() + "</center>");
        insertItem(label);
        insertSeparator();

        insertItem("Detailed info", this, SLOT(slotInfo()));
    }
    if(flags & ToList)
        insertItem("Add to current list", this, SLOT(slotAddToList()));
    if(flags & Delete)
        insertItem("Remove", this, SLOT(slotRemove()));

    if(item->itemClass() == Item::KanjiItem)
    {
        insertItem("Insert into search string", this, SLOT(slotToSearch()));

        insertItem("Radicals to kanji search", this, SLOT(slotSetTheseRadicals()));
    }
}

void ItemPopup::slotToSearch()
{
    emit focusSearch(_item->kanji()[0]);
}

void ItemPopup::slotInfo()
{
    emit showItem(_item);
}

void ItemPopup::slotAddToList()
{
    emit addToList(_item);
}


void ItemPopup::slotSetTheseRadicals()
{
    emit setTheseRadicals(_item->kanji()[0]);
}


void ItemPopup::slotRemove()
{
    emit remove(_id);
}

