/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qradiobutton.h>
#include "itemsettingsdialog.h"

using std::cerr;

ItemSettingsDialog::ItemSettingsDialog(QWidget *parent, const char *name)
 : ItemSettingsDialogBase(parent, name, true, 0)
{
	connect(__defaults, SIGNAL(clicked()), this, SLOT(slotSetListDefaultsClicked()));
	connect(__custom, SIGNAL(clicked()), this, SLOT(slotSetCustomClicked()));
}	


ItemSettingsDialog::~ItemSettingsDialog()
{
}

void ItemSettingsDialog::slotSetListDefaultsClicked()
{
	setCustomEnabled(false);
}

void ItemSettingsDialog::slotSetCustomClicked()
{
	setCustomEnabled(true);
}

void ItemSettingsDialog::setCustomEnabled(bool e)
{
	__kanjiToKana->setEnabled(e);
	__kanjiToMeaning->setEnabled(e);
	__kanaToKanji->setEnabled(e);
	__kanaToMeaning->setEnabled(e);
	__meaningToKanji->setEnabled(e);
	__meaningToKana->setEnabled(e);
}

void ItemSettingsDialog::setItem(Item *item, int defSkills)
{
	_defSkills = defSkills;
	int skills = item->box()->skills();

	__kanjiToKana->setChecked(skills & (1 << Box::KanjiToKana));
	__kanjiToMeaning->setChecked(skills & (1 << Box::KanjiToMeaning));
	__kanaToKanji->setChecked(skills & (1 << Box::KanaToKanji));
	__kanaToMeaning->setChecked(skills & (1 << Box::KanaToMeaning));
	__meaningToKanji->setChecked(skills & (1 << Box::MeaningToKanji));
	__meaningToKana->setChecked(skills & (1 << Box::MeaningToKana));
	
	if(skills & (1 << Box::ListDefaults))
	{
		setCustomEnabled(false);
		__defaults->setChecked(true);
	}
	else
	{
		setCustomEnabled(true);
		__custom->setChecked(true);
	}

	_item = item;
}

void ItemSettingsDialog::accept()
{
	if(!__custom->isChecked())
	{
		_item->box()->setSkills(_defSkills | (1 << Box::ListDefaults));
	}
	else
	{
		_item->box()->setSkills((__kanjiToKana->isChecked() << Box::KanjiToKana) |
							   (__kanjiToMeaning->isChecked() << Box::KanjiToMeaning) |
							   (__kanaToKanji->isChecked() << Box::KanaToKanji) |
							   (__kanaToMeaning->isChecked() << Box::KanaToMeaning) |
							   (__meaningToKanji->isChecked() << Box::MeaningToKanji) |
						       (__meaningToKana->isChecked() << Box::MeaningToKana));
	}
		

	QDialog::accept();
}
