/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "jpnlineedit.h"

JpnLineEdit::JpnLineEdit( QWidget *parent, const char *name) : QLineEdit(parent, name)
{
    installEventFilter(this);
    connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(slotConvertText(const QString&)));
    _mode = Hiragana;
}

JpnLineEdit::~JpnLineEdit()
{}

void JpnLineEdit::setInputMode(InputMode mode)
{
    slotTrimText();

    if(_mode == Romaji)
    {
        _mode = mode;    
        _engString = text();
        if(mode != Romaji)
            setText(_kanaString);
    }
    else if(_mode != Romaji)
    {
        _mode = mode;
        _kanaString = text();
        if(mode == Romaji)
            setText(_engString);
    }

}

void JpnLineEdit::slotConvertText(const QString& str)
{
    if(str.isNull() || _mode == Romaji)
        return;

    int pos = cursorPosition();
        
    if(_mode == Hiragana)
        setText(_conv.romajiToKana(str));
    else if(_mode == Katakana)
        setText(_conv.romajiToKana(str,false));

    setCursorPosition(pos);
}

bool JpnLineEdit::eventFilter(QObject *, QEvent *e)
{
    if(e->type() == QEvent::KeyPress)
    {
        QKeyEvent *k = (QKeyEvent *)e;
        if(k->key() == Key_Up || k->key() == Key_Down)
        {
            emit(keyPressed(k->key()));
            return true;
        }
        else if(k->key() == Key_Return)
        {
            slotTrimText();
            return false;
        }
        emit keyPressed();
    }

    return false;
}


void JpnLineEdit::slotTrimText()
{
    if(text().right(1) != "n" || _mode == Romaji)
        return;

    QString temp = text();
    if(_mode == Hiragana)
        temp[temp.length()-1] = QChar(0x3093);
    else
        temp[temp.length()-1] = QChar(0x30f3);
    setText(temp);
}

void JpnLineEdit::initialize(const QString& fileName)
{
    _conv.initialize(fileName);
}

Convert JpnLineEdit::_conv;
