/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef JPNWORDENTRY_H
#define JPNWORDENTRY_H

#include <qcombobox.h>
#include "convert.h"
#include "jpnlineedit.h"
#include "jpnwordentrywidget.h"
#include "dictionary.h"

class JpnWordEntry : public JpnWordEntryWidget
{
    Q_OBJECT
public:

    JpnWordEntry(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
    ~JpnWordEntry();

    void initialize(Convert *convert);

    void insert(QChar ch);
    QString text() const {return __input->text();};
    int kind() const {return __input->inputMode();};

    void setBusy(bool busy);
	void alert(const QString& message);

signals:
    void keyPressed(int);
    void returnPressed();
    void setSearchMode(Dictionary::SearchMode);
	void cancelClicked();

public slots:
	void slotCancelClicked();
    void slotTrimText();
    void slotFocus();
    void slotFocus(QChar ch);
	void slotSetProgress(const QString& message, float progress);

private:
	void updateMenuCharacter();	
    QPopupMenu *_inputMenu;
    QPopupMenu *_searchModeMenu;
    int _beginningId, _exactId, _anywhereId, _endId, _hiraganaId, _katakanaId, _romajiId;
	int _currentSearchModeId;

private slots:

	void slotFontsChanged();
	void slotIconsChanged();

    void slotInputModeActivated(int id);
    void slotSearchModeActivated(int id);
    void slotKeyPressed(int key);
	void slotShowProgressBar();
	void slotHideProgressBar();

};

#endif
