/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qcursor.h>
#include "kanjibrowser.h"
#include "kanjidic.h"
#include "itempopup.h"


using namespace std;

KanjiBrowser::KanjiBrowser(QWidget *parent, const char *name)
        : HypertextBrowser(parent, name)
{
    connect(this, SIGNAL(linkClicked(const QString&, bool)), this, SLOT(slotLinkClicked(const QString&, bool)));


	bool _vertical = false;
    _pop = new ItemPopup(this);
	
}

KanjiBrowser::~KanjiBrowser()
{
}

void KanjiBrowser::setOrientation(bool vertical)
{
	_vertical = vertical;
	updateText();
}

void KanjiBrowser::clear()
{
	setText("");
}

void KanjiBrowser::setKanji(const QString& str)
{
	_kanji = str;
	updateText();
}

void KanjiBrowser::addKanji(QChar ch)
{
	_kanji += ch;
	updateText();
}

void KanjiBrowser::updateText()
{
	if(_kanji.length() == 0)
	{
		setText(tr("Empty"));
		return;
	}
	QString str("<p><largekana>");

	for(int i=0; i<_kanji.length(); i++)
	{
		str += QString("<a href=K%1>%2</a>").arg(_kanji[i]).arg(_kanji[i]);
	}

	str += "</largekana></p>";

	setText(str);

}


void KanjiBrowser::slotLinkClicked(const QString& str, bool mouse)
{
    QPoint pos = mouse ? QCursor::pos() : QWidget::mapToGlobal(QPoint(width() / 2, height() / 2));   

    if(str[0] == 'K')
    {
        Kanji kanji =  glKanjiDic->getKanji(str[1]);
	   	_pop->setItem(&kanji, ItemPopup::PopupFlags(ItemPopup::ToDetails | ItemPopup::ToList), 0);
	    _pop->exec(pos, 0);
        return;
    }

}

