/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qlabel.h>
#include <qcursor.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qbuttongroup.h>
#include "hcr.h"
#include "item.h"
#include "itempopup.h"
#include "kanjilookup.h"
#include "radicalslookup.h"
#include "kanjisearch.h"
#include "configmanager.h"
#include "fonts.h"
#include "kanjibrowser.h"

using namespace std;

KanjiLookup::KanjiLookup(QWidget *parent, const char *name, WFlags f) : KanjiLookupWidget(parent, name, f)
{
    connect(__hcr, SIGNAL(kanjiSelected(QChar)), this, SLOT(slotKanjiSelected(QChar)));
    connect(__hcr, SIGNAL(kanjiHeld(QChar)), this, SLOT(slotKanjiHeld(QChar)));
    connect(__kanjiSearch, SIGNAL(kanjiSelected(QChar)), this, SLOT(slotKanjiSelected(QChar)));
	connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(slotSaveState()));
	connect(__drawButton, SIGNAL(clicked()), this, SLOT(slotShowHCR()));
	connect(__searchButton, SIGNAL(clicked()), this, SLOT(slotShowSearch()));
	connect(__recentButton, SIGNAL(clicked()), this, SLOT(slotShowRecent()));
	connect(__drawButton_2, SIGNAL(clicked()), this, SLOT(slotShowHCR()));
	connect(__searchButton_2, SIGNAL(clicked()), this, SLOT(slotShowSearch()));
	connect(__recentButton_2, SIGNAL(clicked()), this, SLOT(slotShowRecent()));

	connect(glFonts, SIGNAL(fontsChanged()), this, SLOT(slotFontsChanged()));
	connect(glFonts, SIGNAL(iconsChanged()), this, SLOT(slotIconsChanged()));

    _pop = new ItemPopup(this);

    _maxHistory = 30;

    QString str;
    str = glConfigManager->readEntry("RecentKanji");
    __recent->setKanji(str);
	_history = str;
	showPage(__hcr);

	if(qApp->desktop()->width() < qApp->desktop()->height())
		__vButtons->hide();
	else
		__hButtons->hide();
}

KanjiLookup::~KanjiLookup()
{}

void KanjiLookup::slotIconsChanged()
{
	QString str;

	if(glFonts->iconSize() == Fonts::ICONSIZE_Small)	
		str = "small_";
	else
		str = "large_";

	__searchButton->setPixmap(glFonts->imagePath() + str + "search.png");	
	__searchButton_2->setPixmap(glFonts->imagePath() + str + "search.png");	
	__recentButton->setPixmap(glFonts->imagePath() + str + "recent.png");	
	__recentButton_2->setPixmap(glFonts->imagePath() + str + "recent.png");	
	__drawButton->setPixmap(glFonts->imagePath() + str + "draw.png");	
	__drawButton_2->setPixmap(glFonts->imagePath() + str + "draw.png");	
}

void KanjiLookup::slotFontsChanged()
{
	__recent->updateFonts();
}

void KanjiLookup::slotShowHCR()
{
	showPage(__hcr);
}

void KanjiLookup::slotShowRecent()
{
	showPage(__recent);
}

void KanjiLookup::slotShowSearch()
{
	showPage(__kanjiSearch);
}

void KanjiLookup::slotKanjiSelected(QChar ch)
{
    if(_history.contains(ch) == 0)
    {
        if(_history.length() == _maxHistory)
            _history.remove(0, 1);
        _history.append(ch);
        __recent->setKanji(_history);
    }
    emit(kanjiSelected(ch));
}

void KanjiLookup::slotKanjiHeld(QChar ch)
{
    Kanji k = glKanjiDic->getKanji(ch);

    _pop->setItem(&k, ItemPopup::PopupFlags(ItemPopup::ToDetails | ItemPopup::ToList));

    _pop->exec(QCursor::pos(), 0);
}

void KanjiLookup::slotSetTheseRadicals(QChar ch)
{
	showPage(__kanjiSearch);
    __kanjiSearch->slotSetTheseRadicals(ch);
}

void KanjiLookup::showPage(QWidget *widget)
{
	if(widget == __kanjiSearch)
	{
		__kanjiSearch->show();
		__recent->hide();
		__hcr->hide();
	}
	else if(widget == __hcr)
	{
		__hcr->show();
		__recent->hide();
		__kanjiSearch->hide();
	}
	else if(widget == __recent)
	{
		__recent->show();
		__hcr->hide();
		__kanjiSearch->hide();
	}

}

void KanjiLookup::slotSaveState()
{
	glConfigManager->setGroup("General");
    glConfigManager->writeEntry("RecentKanji", __recent->kanji());
}

void KanjiLookup::resizeEvent(QResizeEvent *e)
{
	if(qApp->desktop()->width() < qApp->desktop()->height())
	{
		__vButtons->hide();
		__hButtons->show();
	}
	else
	{
		__hButtons->hide();
		__vButtons->show();
	}

	QWidget::resizeEvent(e);

}


