/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qobjectlist.h>
#include <qcursor.h>
#include "kanjisearch.h"
#include "item.h"
#include "fonts.h"

using namespace std;

KanjiSearch::KanjiSearch(QWidget *parent, const char *name) : QWidgetStack(parent, name)
{
	_vert = new VKanjiSearchWidget(this);
	_horiz = new HKanjiSearchWidget(this);

	addWidget(_horiz, 1);
	addWidget(_vert, 2);

	if(qApp->desktop()->height() > qApp->desktop()->width())
		_currW = _vert;
	else
		_currW = _horiz;

	raiseWidget(_currW);

    QObjectList *list = queryList("QCheckBox");

    for(int i=0; i<list->count(); i++)
	    connect((QCheckBox *)list->at(i), SIGNAL(released()), this, SLOT(slotUpdateWidgets()));

	setupWidget(_horiz);
	setupWidget(_vert);

    _pop = new ItemPopup(this);

	if(_currW == _vert)
    	updateWidgets(_vert);
	else
    	updateWidgets(_horiz);

	connect(glFonts, SIGNAL(fontsChanged()), this, SLOT(slotFontsChanged()));
}

KanjiSearch::~KanjiSearch()
{
}

void KanjiSearch::slotFontsChanged()
{
	_vert->__browser->updateFonts();
	_horiz->__browser->updateFonts();

}

void KanjiSearch::swapWidgets()
{

	if(_currW == _vert)
	{
		_currW = _horiz;
		copyWidgetContent(_vert, _horiz);
	}
	else
	{
		_currW = _vert;
		copyWidgetContent(_horiz, _vert);
	}

	raiseWidget(_currW);
}

void KanjiSearch::slotSearchClicked()
{
	if(_currW == _vert)
		searchClicked(_vert);
	else
		searchClicked(_horiz);

}

void KanjiSearch::slotKanjiHeld(QChar ch)
{
    Kanji k = glKanjiDic->getKanji(ch);
    _pop->setItem(&k, ItemPopup::PopupFlags(ItemPopup::ToList | ItemPopup::ToDetails));
    _pop->exec(QCursor::pos(), 0);
}

void KanjiSearch::slotUpdateWidgets()
{
	if(_currW == _vert)
		updateWidgets(_vert);
	else
		updateWidgets(_horiz);

}


void KanjiSearch::slotRadicalsClicked()
{
	if(_currW == _vert)	
		radicalsClicked(_vert);
	else
		radicalsClicked(_horiz);
}


void KanjiSearch::slotReadingsClicked()
{

	if(_currW == _horiz)
		readingsClicked(_horiz);
	else
		readingsClicked(_vert);

}


void KanjiSearch::slotSetTheseRadicals(QChar ch)
{
	if(_currW == _vert)
	{
	    _vert->__radicals->setText(glKanjiDic->getRadicals(ch));
		_vert->__useRadicals->setChecked(true);
		updateWidgets(_vert);
	}
	else
	{
	    _horiz->__radicals->setText(glKanjiDic->getRadicals(ch));
		_horiz->__useRadicals->setChecked(true);
		updateWidgets(_horiz);
	}

}

void KanjiSearch::resizeEvent(QResizeEvent *e)
{
	if(qApp->desktop()->height() > qApp->desktop()->width())
	{
		if(_currW == _horiz)
			swapWidgets();
	}
	else if(_currW == _vert)
	{
		swapWidgets();
	}

	_currW->resize(width(), height());

	QWidget::resizeEvent(e);

	_currW->updateGeometry();

}


