/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef KANJISEARCH_H
#define KANJISEARCH_H

#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include "hvwidgetstack.h"
#include "itempopup.h"
#include "vkanjisearchwidget.h"
#include "hkanjisearchwidget.h"
#include "readingsdialog.h"
#include "radicalsdialog.h"
#include "kanjiquery.h"
#include "kanjibrowser.h"


class KanjiSearch : public QWidgetStack
{
Q_OBJECT

public:
    KanjiSearch(QWidget *parent = 0, const char *name = 0);
    ~KanjiSearch();

signals:
    void kanjiSelected(QChar);
    void showItem(const Item *);
    void addToListItem(const Item *);

public slots:
    void slotSetTheseRadicals(QChar ch);

private slots:
    void slotKanjiHeld(QChar ch);
	void slotUpdateWidgets();
    void slotSearchClicked();
    void slotRadicalsClicked();
    void slotReadingsClicked();
    void slotFontsChanged();
private:

	void resizeEvent(QResizeEvent *e);
	void swapWidgets();

	template<class T> void setupWidget(T *t)
	{
	   	connect(t->__search, SIGNAL(clicked()), this, SLOT(slotSearchClicked()));
	   	connect(t->__radicals, SIGNAL(clicked()), this, SLOT(slotRadicalsClicked()));
	   	connect(t->__readings, SIGNAL(clicked()), this, SLOT(slotReadingsClicked()));
		connect(t->__useStrokes, SIGNAL(clicked()), this, SLOT(slotUpdateWidgets()));
		connect(t->__useReadings, SIGNAL(clicked()), this, SLOT(slotUpdateWidgets()));
		connect(t->__useDeviation, SIGNAL(clicked()), this, SLOT(slotUpdateWidgets()));
		connect(t->__useRadicals, SIGNAL(clicked()), this, SLOT(slotUpdateWidgets()));
	};

	template<class T> void updateWidgets(T *t)
	{
    	t->__strokes->setEnabled(t->__useStrokes->isChecked());
	    t->__deviation->setEnabled(t->__useDeviation->isChecked() && t->__useStrokes->isChecked());
	    t->__radicals->setEnabled(t->__useRadicals->isChecked());
	    t->__readings->setEnabled(t->__useReadings->isChecked());
	    t->__search->setEnabled(t->__useStrokes->isChecked() || (t->__useRadicals->isChecked()) || t->__useReadings->isChecked());
	   	t->__useDeviation->setEnabled(t->__useStrokes->isChecked());
	};

	template<class S, class D> void copyWidgetContent(S *source, D *dest)
	{
		dest->__strokes->setValue(source->__strokes->value());
		dest->__deviation->setValue(source->__deviation->value());
		dest->__radicals->setText(source->__radicals->text());
		dest->__readings->setText(source->__readings->text());		dest->__useRadicals->setChecked(source->__useRadicals->isChecked());
		dest->__useReadings->setChecked(source->__useReadings->isChecked());
		dest->__useStrokes->setChecked(source->__useStrokes->isChecked());
		dest->__useDeviation->setChecked(source->__useDeviation->isChecked());
		dest->__browser->setKanji(source->__browser->kanji());

        if(_on.length() > 0 && _name.length() > 0)
			dest->__readings->setText(_on + "+" + _name);
		
        else
            dest->__readings->setText(_on + _name);


		updateWidgets(dest);
	};

	template<class T> void readingsClicked(T *t)
	{
	    ReadingsDialog *req = new ReadingsDialog(this);
		req->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));
	    req->setReadings(_on, _name, _meaning);
		QString readings;

	    if(req->exec())
	    {
        	_on = req->on();
	        _name = req->name();
   	    	_meaning = req->meaning();

	        if(_on.length() > 0 && _name.length() > 0)
				t->__readings->setText(_on + "+" + _name);
	        else
    	        t->__readings->setText(_on + _name);
		}
    };

	template<class T> void radicalsClicked(T *t)
	{
	    RadicalsDialog *req = new RadicalsDialog(this);
		req->resize(qApp->desktop()->width() * (2./3.), qApp->desktop()->height() * (2./3.));

    	if(t->__radicals->text() != "-")
    	    req->setRadicals(t->__radicals->text());
	    if(req->exec())
       		t->__radicals->setText(req->radicals());
	};
	

	template<class T> void searchClicked(T *t)
	{
	    KanjiQuery kq;
	    QString res;

	    if(t->__useStrokes->isChecked() && t->__strokes->value() > 0)
	    {
   	     if(t->__useDeviation->isChecked())
   	         kq.setStrokes(t->__strokes->value(), t->__deviation->value());
	        else
            kq.setStrokes(t->__strokes->value());
    	}

	    if(t->__useRadicals->isChecked() && t->__radicals->text() != "-")
	        kq.setRadicals(t->__radicals->text());

	    if(t->__useReadings->isChecked())
    	    kq.setReadings(_on, _name, _meaning);

	    res = kq.search();

       	t->__browser->setKanji(res);
	}

    QString _on;
    QString _name;
    QString _meaning;
    ItemPopup *_pop;
	HKanjiSearchWidget *_horiz;
	VKanjiSearchWidget *_vert;
	QWidget *_currW;
	
};

#endif
