/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>
#include <qpainter.h>
#include "progressbar.h"

using namespace std;

ProgressBar::ProgressBar(QWidget *parent, const char *name)
 : QWidget(parent, name)
{
    _text = tr("Ready");
    _prog = 0;
    _alert = 0;
    _alertCount = 0;
    _timer = new QTimer(this);
	setBackgroundMode(NoBackground);
    connect(_timer, SIGNAL(timeout()), this, SLOT(slotAlertTimer()));
}


ProgressBar::~ProgressBar()
{
}


void ProgressBar::slotSetText(const QString& text, float prog=0)
{
    _alert = false;
    _text = text;
    _prog = prog;
    update();
}

void ProgressBar::slotSetText(const QString& text)
{
    _text = text;
    _prog = 0;
    update();
}

void ProgressBar::slotAlert()
{
    _alert = true;
    _alertToggle = true;
    _alertCount = 0;
    
    _timer->start(200, true);
}

void ProgressBar::slotAlertTimer()
{
    _alertCount++;
    
    if(_alertCount > 8)
    {
        _alertCount = 0;
        _alertToggle = false,
        _alert = 0;
        update();
		emit alertDone();
        return;
    }

    _alertToggle = !_alertToggle;
    
    update();    
    _timer->start(200, true);
    
}

void ProgressBar::paintEvent(QPaintEvent *)
{
    QPainter p;
    p.begin(this);
    
    if(_alert)
    {
        if(_alertToggle)
            p.fillRect(0, 0, width(),  height(), colorGroup().mid());
		else
            p.fillRect(0, 0, width(),  height(), colorGroup().background());
    }   
    else
	{
		int x = int(float(width()) * _prog);
        p.fillRect(0, 0, x, height(), colorGroup().mid());
        p.fillRect(x, 0, width()-x, height(), colorGroup().background());
    }
    p.drawText(0, 0, width(), height(), AlignLeft | AlignVCenter, _text);
    
    p.end();
}
