/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qcombobox.h>
#include "kanjidic.h"
#include "radicalsdialog.h"
#include "buttongrid.h"
#include "horizbuttons.h"

RadicalsDialog::RadicalsDialog(QWidget *parent, const char *name)
 : RadicalsDialogBase(parent, name, true, 0)
{
    _selRadButtons.setAutoDelete(true);
    
    for(int i=1; i < 18; i++)
        __strokes->insertItem(QString().setNum(i));
    
    connect(__strokes, SIGNAL(activated(int)), this, SLOT(slotPopulate(int)));
    connect(__grid, SIGNAL(selectionChanged()), this, SLOT(slotRadicalsChanged()));
    connect(__selectedButtons, SIGNAL(buttonClicked(QChar)), this, SLOT(slotSelectedRadicalClicked(QChar)));

    __grid->setToggleButtons(true);

    slotPopulate(0);
}


RadicalsDialog::~RadicalsDialog()
{
}


void RadicalsDialog::slotRadicalsChanged()
{
    QString str = __grid->selected();
    __selectedButtons->setButtons(str);  
    
}

void RadicalsDialog::slotSelectedRadicalClicked(QChar ch)
{
    __grid->unselect(ch);
    __selectedButtons->setButtons(__grid->selected());
}


void RadicalsDialog::slotPopulate(int index)
{
    QString str = glKanjiDic->getRadicalsByStrokes(index+1);

    __grid->setButtons(str);

}

void RadicalsDialog::setRadicals(const QString& str)
{
    __selectedButtons->setButtons(str);
    __grid->setSelected(str);
}

QString RadicalsDialog::radicals()
{
    return __grid->selected();
}

