/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qspinbox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include "kanjidic.h"
#include "radicalslookup.h"
#include "radicalgrid.h"
#include "horizbuttons.h"
#include "vertbuttons.h"

using namespace std;

RadicalsLookup::RadicalsLookup(QWidget *parent, const char *name)
 : RadicalsLookupWidget(parent, name)
{
    _selRadButtons.setAutoDelete(true);
    
    radicalGrid->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

    for(int i=1; i < 18; i++)
        strokesCB->insertItem(QString().setNum(i));
    
    connect(strokesCB, SIGNAL(activated(int)), this, SLOT(slotPopulate(int)));
    connect(radicalGrid, SIGNAL(selectionChanged()), this, SLOT(slotRadicalsChanged()));
    connect(selectedButtons, SIGNAL(buttonClicked(QChar)), this, SLOT(slotSelectedRadicalClicked(QChar)));
    connect(candButtons, SIGNAL(buttonClicked(QChar)), this, SIGNAL(kanjiSelected(QChar)));
    connect(candButtons, SIGNAL(buttonHeld(QChar)), this, SIGNAL(kanjiHeld(QChar)));
    connect(findB, SIGNAL(clicked()), this, SLOT(slotSearch()));
    
    slotPopulate(0);
}

RadicalsLookup::~RadicalsLookup()
{
}

void RadicalsLookup::slotSearch()
{
    QString results = glKanjiDic->getKanjiByRadicals(radicalGrid->selected(), totalStrokesSB->value());
        
    candButtons->setButtons(results);
}

void RadicalsLookup::slotRadicalsChanged()
{
    QString str = radicalGrid->selected();
    selectedButtons->setButtons(str);  
    
}

void RadicalsLookup::slotSelectedRadicalClicked(QChar ch)
{
    radicalGrid->unselect(ch);
    selectedButtons->setButtons(radicalGrid->selected());
}


void RadicalsLookup::slotPopulate(int index)
{

    QString str = glKanjiDic->getRadicalsByStrokes(index+1);

    radicalGrid->setButtons(str);

}

void RadicalsLookup::slotSetTheseRadicals(QChar ch)
{
    QString str = glKanjiDic->getRadicals(ch);
    selectedButtons->setButtons(str);
    radicalGrid->setSelected(str);
}


