/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "jpnlineedit.h"
#include <qcheckbox.h>
#include <qobjectlist.h>
#include "readingsdialog.h"


ReadingsDialog::ReadingsDialog(QWidget *parent, const char *name)
 : ReadingsDialogBase(parent, name, true, 0)
{
    QObjectList *list = queryList("QCheckBox");

    for(int i=0; i<list->count(); i++)
        connect((QCheckBox *)list->at(i), SIGNAL(released()), this, SLOT(slotUpdateWidgets()));


    __on->setInputMode(JpnLineEdit::Katakana);
    __name->setInputMode(JpnLineEdit::Hiragana);

    slotUpdateWidgets();
}


ReadingsDialog::~ReadingsDialog()
{
}

void ReadingsDialog::setReadings(const QString& on, const QString& name, const QString& meaning)
{
    __on->setText(on);
    __name->setText(name);
    __meaning->setText(meaning);

    if(__on->text().length() > 0)
        __useOn->setChecked(true);
    if(__name->text().length() > 0)
        __useName->setChecked(true);
    if(__meaning->text().length() > 0)
        __useMeaning->setChecked(true);

    slotUpdateWidgets();
}

QString ReadingsDialog::on()
{
    if(__useOn->isChecked())
        return __on->text();
    else
        return "";
}

QString ReadingsDialog::name()
{
    if(__useName->isChecked())
        return __name->text();
    else
        return "";
}

QString ReadingsDialog::meaning()
{
    if(__useMeaning->isChecked())
        return __meaning->text();
    else
        return "";
}


void ReadingsDialog::slotUpdateWidgets()
{

    __on->setEnabled(__useOn->isChecked());
    __name->setEnabled(__useName->isChecked());
    __meaning->setEnabled(__useMeaning->isChecked());

}
