/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <qcstring.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <qtextbrowser.h>
#include <qtabwidget.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include "results.h"
#include "search.h"
#include "jpnwordentry.h"
#include "itemlistview.h"
#include "kanjilookup.h"

using std::cerr;


Search::Search(QWidget *parent, const char *name, WFlags f)
        : SearchWidget(parent, name, f)
{

    connect(__input, SIGNAL(returnPressed()), this, SLOT(slotSearch()));
    connect(__input, SIGNAL(setSearchMode(Dictionary::SearchMode)), this, SLOT(slotSetSearchMode(Dictionary::SearchMode)));
    connect(__lookup, SIGNAL(kanjiSelected(QChar)), this, SLOT(slotInsertChar(QChar)));
	connect(__input, SIGNAL(cancelClicked()), this, SLOT(slotCancel()));
    connect(glDictManager, SIGNAL(searchProgress(const QString&, float)), __input, SLOT(slotSetProgress(const QString&, float)));


}

Search::~Search()
{
}

void Search::slotFocusSearch()
{
    __input->slotFocus();
}

void Search::slotFocusSearch(QChar ch)
{
    __input->slotFocus(ch);
}

void Search::slotInputKeyPressed()
{
}

void Search::slotInsertChar(const Kanji& kanji)
{
    __input->insert(kanji.kanji()[0]);
}

void Search::slotInsertChar(QChar ch)
{
    __input->insert(ch);
}

void Search::slotSearch()
{
    bool J2E = true;
	QList<const DictWord> *res = new QList<const DictWord>;

    if(__input->text().isEmpty())
        return;

    __input->slotTrimText();

	__input->setBusy(true);

//check string for Japanese characters instead of input mode

    if(__input->kind() != JpnLineEdit::Romaji)
        glDictManager->searchAllJ2E(__input->text(), res);
    else
    {
        glDictManager->searchAllE2J(__input->text().utf8(), res);
        J2E = false;
    }

	__input->setBusy(false);

	if(res->count() == 0)
	{
		__input->alert(tr("No matches found"));
		return;
	}

    __results->newResult(glDictManager->searchMode(), QStringList(__input->text()), res, J2E, false);

    focusResults();
}

void Search::slotCancel()
{
    glDictManager->cancelSearch();
}

void Search::focusResults()
{
    __tabs->setCurrentPage(1);
    __results->setFocus();
}

void Search::slotSetSearchMode(Dictionary::SearchMode mode)
{
    glDictManager->setSearchMode(mode);
}

void Search::slotSetTheseRadicals(QChar ch)
{
    __tabs->setCurrentPage(0);
    __lookup->slotSetTheseRadicals(ch);
}
