/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <time.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlistview.h>
#include "trainselect.h"
#include "listmanager.h"

TrainSelect::TrainSelect(QWidget *parent, const char *name)
        : TrainSelectWidget(parent, name)
{
//	_timer = new QTimer(this);

	connect(__selAll, SIGNAL(clicked()), this, SLOT(slotSelAllClicked()));
	connect(__unselAll, SIGNAL(clicked()), this, SLOT(slotUnselAllClicked()));
	connect(__invertSel, SIGNAL(clicked()), __list, SLOT(invertSelection()));
	connect(__start, SIGNAL(clicked()), this, SLOT(slotStartClicked()));
	
	connect(glListManager, SIGNAL(doneLoading()), this, SLOT(slotUpdateList()));
	connect(glListManager, SIGNAL(changed()), this, SLOT(slotUpdateList()));
//	connect(_timer, SIGNAL(timeout()), this, SLOT(slotUpdateList()));
	connect(__update, SIGNAL(clicked()), this, SLOT(slotUpdateList()));

//	_timer->start(60000);

}

TrainSelect::~TrainSelect()
{
}

void TrainSelect::slotSelAllClicked()
{
	__list->selectAll(true);
}

void TrainSelect::slotUnselAllClicked()
{
	__list->selectAll(false);
}

void TrainSelect::slotStartClicked()
{
	_itemsToTrain.clear();

	QListViewItem *lvitem = __list->firstChild();

	while(lvitem)
	{
		if(__list->isSelected(lvitem))
		{
			ItemList *list = glListManager->getListByName(lvitem->text(0));
			list->setLocked(true);
			for(int j=0; j < list->count(); j++)
			{
				Item *item = (*list)[j];
				int skills;
				if(item->box()->skills() & (1 << Box::ListDefaults))
					skills = list->defaultSkills();
				else
					skills = item->box()->skills();

				for(int skill=0; skill<6; skill++)
				{
					if(((skill == Box::MeaningToKanji) || (skill == Box::KanaToKanji)|| (skill == Box::KanjiToKana)|| (skill == Box::KanjiToMeaning)) && (item->kanji().length() == 0))
						continue;
					if((skills & (1 << skill)) == 0)
						continue;
					if(item->box()->timeLeft((Box::Skills)skill) < 1)
						_itemsToTrain.append((Box::Skills)skill, item, list);
				}
			}
		}
		lvitem = lvitem->nextSibling();
	}

	if(_itemsToTrain.count() == 0)
		return;

	if(__random->isChecked())
		_itemsToTrain.randomizeOrder();

	emit showBoxTrain();

}

void TrainSelect::showEvent(QShowEvent *)
{
	slotUpdateList();
}

void TrainSelect::slotUpdateList()
{
	__list->clear();

    if(glListManager->count() == 0)
        return;

	QList<ItemList> lists = glListManager->lists();

	QListIterator<ItemList> it(lists);

	for(; it.current(); ++it)
	{
		new QListViewItem(__list, it.current()->name(), QString().setNum(it.current()->count()), QString().setNum(it.current()->dueCount()));
	}

}

TrainList::TrainList()
{
	_items.setAutoDelete(true);
	// note that this deletes the TrainItems created in this class, not the items
	// that the TrainItems point to
}

TrainList::~TrainList()
{
}



void TrainList::randomizeOrder()
{
	QList<TrainItem> rndItems;

	srand(time(0));

	while(_items.count())
	{
		rndItems.append(_items.take(rand() % _items.count()));
	}

	_items = rndItems;

}

void TrainList::append(Box::Skills skill, Item *item, ItemList *ownerList)
{
	_items.append(new TrainItem(skill, item, ownerList));
}

void TrainItem::removeSkill()
{
	// Remember that Box::skills() is a bitmap with the skills of the item, while
	// TrainItem::_skill an integer with the skill of this TrainItem. This code unsets the bits
	// for ListDefaults AND this object's _skill in the Item pointed to by box().
	box()->setSkills(box()->skills() & (~((1 << Box::ListDefaults) | (1 << _skill))));

}
