/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef TRAINSELECT_H
#define TRAINSELECT_H

#include "trainselectwidget.h"
#include "itemlist.h"
//#include "qtimer.h"

class TrainItem
{
public:

	TrainItem(Box::Skills skill, Item *item, ItemList *ownerList) {_skill = skill; _item = item; _ownerList = ownerList;};
	~TrainItem() {};

	QString kanji() const {return _item->kanji();};
	QString kana() const {return _item->kana();};
	QString english() const {return _item->english();};

	void demote() {_ownerList->setChanged(true); box()->demote(_skill);};
	void promote() {_ownerList->setChanged(true); box()->promote(_skill);};

	ItemList *ownerList() {return _ownerList;};

	Item::ItemClasses itemClass() const {return _item->itemClass();};

	Box::Skills skill() const {return _skill;};
	int rank() const {return _item->box()->rank(_skill);};
	QString dateString() const {return _item->box()->dateString(_skill);};
	Box *box() {return _item->box();};
	void removeSkill();

	
private:
	Box::Skills _skill;
	Item *_item;
	ItemList *_ownerList;
	
};


class TrainList
{
public:

	TrainList();
	~TrainList();

	void randomizeOrder();

	int count() const {return _items.count();};
	void clear() {_items.clear();};
	void append(Box::Skills skill, Item *item, ItemList *ownerList);

	TrainItem *operator[](int i) {return _items.at(i);};

private:
	QList<TrainItem> _items;

};


class TrainSelect : public TrainSelectWidget
{
    Q_OBJECT
public:

    TrainSelect(QWidget *parent = 0, const char *name = 0);
    ~TrainSelect();

	TrainList *itemsToTrain() { return &_itemsToTrain; };

signals:
	void showBoxTrain();

public slots:
	void slotUpdateList();

private slots:
	void slotSelAllClicked();
	void slotUnselAllClicked();
	void slotStartClicked();

private:
	TrainList _itemsToTrain;
	void showEvent(QShowEvent *);
//	QTimer *_timer;

};
	
#endif
