/*
    JARDS, Japanese Reference and Drilling System.
    Copyright (C) 2006  Johnny Andersson (e-mail: andersson.johnny@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <qapplication.h>
#include "vertbuttons.h"

VertButtons::VertButtons(QWidget *parent, const char *name)
        : VertButtonsWidget(parent, name)
{
    scrollV->setVScrollBarMode(QScrollView::AlwaysOff);
    scrollV->setHScrollBarMode(QScrollView::AlwaysOff);

    _vBox = new QVBox(scrollV->viewport());
    scrollV->addChild(_vBox);

    _buttons.setAutoDelete(true);

    connect(upB, SIGNAL(clicked()), this, SLOT(slotUpClicked()));
    connect(downB, SIGNAL(clicked()), this, SLOT(slotDownClicked()));
}



VertButtons::~VertButtons()
{
}

void VertButtons::slotUpClicked()
{
    scrollV->scrollBy(0, -scrollV->height() / 5);
    adjustButtons();
}

void VertButtons::slotDownClicked()
{
    scrollV->scrollBy(0, scrollV->height() / 5);
    adjustButtons();
}

void VertButtons::adjustButtons()
{
    qApp->processEvents();

    if(scrollV->contentsY() != 0)
        upB->setEnabled(true);
    else
        upB->setEnabled(false);

   if( _vBox->height() - scrollV->contentsY() > scrollV->height())
        downB->setEnabled(true);
    else
        downB->setEnabled(false);
}

void VertButtons::resizeEvent(QResizeEvent *e)
{
    adjustButtons();
}

void VertButtons::setButtons(QString labels)
{
    KanjiButton *b;

    _buttons.clear();

    for(int i=0; i < labels.length(); i++)
    {
        b = new KanjiButton(QString(labels[i]), _vBox);
        connect(b, SIGNAL(kanjiClicked(QChar)), this, SIGNAL(buttonClicked(QChar)));
        connect(b, SIGNAL(kanjiHeld(QChar)), this, SIGNAL(buttonHeld(QChar)));
        connect(b, SIGNAL(kanjiClicked(QChar, QButton::ToggleState)), this, SIGNAL(buttonClicked(QChar, QButton::ToggleState)));
        b->resize(b->sizeHint());
        b->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
        _buttons.append(b);
        b->show();
    }

    if(labels.length() != 0)
        _vBox->resize(b->width(), height());
        
//     resize(b->width(), height());

    adjustButtons();

}

